# (c) Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

from oslo_config import cfg

from barbican.common import config
from barbican.common import exception
from barbican.common import utils
from barbican import i18n as u
from barbican.model import models as db_models
from barbican.model import repositories as db_repos

LOG = utils.getLogger(__name__)

LOOKUP_PLUGINS_PREFIX = "secretstore:"


def read_multiple_backends_config():
    """Reads and validates multiple backend related configuration.

    Multiple backends configuration is read only when multiple secret store
    flag is enabled.
    Configuration is validated to make sure that section specific to
    provided suffix exists in service configuration. Also validated that only
    one of section has global_default = True and its not missing.
    """
    conf = config.get_module_config('secretstore')

    parsed_stores = None
    if utils.is_multiple_backends_enabled():
        suffix_list = conf.secretstore.stores_lookup_suffix
        if not suffix_list:
            raise exception.MultipleSecretStoreLookupFailed()

        def register_options_dynamically(conf, group_name):
            store_opt_group = cfg.OptGroup(
                name=group_name, title='Plugins needed for this backend')
            store_opts = [
                cfg.StrOpt('secret_store_plugin',
                           help=u._('Internal name used to identify'
                                    'secretstore_plugin')
                           ),
                cfg.StrOpt('crypto_plugin',
                           help=u._('Internal name used to identify '
                                    'crypto_plugin.')
                           ),
                cfg.BoolOpt('global_default',
                            default=False,
                            help=u._('Flag to indicate if this plugin is '
                                     'global default plugin for deployment. '
                                     'Default is False.')
                            ),
            ]
            conf.register_group(store_opt_group)
            conf.register_opts(store_opts, group=store_opt_group)

        group_names = []
        # construct group names using those suffix and dynamically register
        # oslo config options under that group name
        for suffix in suffix_list:
            group_name = LOOKUP_PLUGINS_PREFIX + suffix
            register_options_dynamically(conf, group_name)
            group_names.append(group_name)

        store_conf = collections.namedtuple('store_conf', ['store_plugin',
                                                           'crypto_plugin',
                                                           'global_default'])
        parsed_stores = []
        global_default_count = 0
        # Section related to group names based of suffix list are always found
        # as we are dynamically registering group and its options.
        for group_name in group_names:
            conf_section = getattr(conf, group_name)
            if conf_section.global_default:
                global_default_count += 1

            store_plugin = conf_section.secret_store_plugin
            if not store_plugin:
                raise exception.MultipleStorePluginValueMissing(conf_section)

            parsed_stores.append(store_conf(store_plugin,
                                            conf_section.crypto_plugin,
                                            conf_section.global_default))

        if global_default_count != 1:
            raise exception.MultipleStoreIncorrectGlobalDefault(
                global_default_count)

    return parsed_stores


def sync_secret_stores(secretstore_manager, crypto_manager=None):
    """Synchronize secret store plugin names between service conf and database

    This method reads secret and crypto store plugin name from service
    configuration and then synchronizes corresponding data maintained in
    database SecretStores table.

    Any new plugin name(s) added in service configuration is added as a new
    entry in SecretStores table. If global_default value is changed for
    existing plugins, then global_default flag is updated to reflect that
    change in database. If plugin name is removed from service configuration,
    then removal is possible as long as respective plugin names are NOT set as
    preferred secret store for a project. If it is used and plugin name is
    removed, then error is raised. This logic is intended to be invoked at
    server startup so any error raised here will result in critical failure.
    """
    if not utils.is_multiple_backends_enabled():
        return

    # doing local import to avoid circular dependency between manager and
    # current utils module
    from barbican.plugin.crypto import manager as cm

    secret_stores_repo = db_repos.get_secret_stores_repository()
    proj_store_repo = db_repos.get_project_secret_store_repository()
    if crypto_manager is None:
        crypto_manager = cm.get_manager()

    def get_friendly_name_dict(ext_manager):
        """Returns dict of plugin internal name and friendly name entries."""
        names_dict = {}
        for ext in ext_manager.extensions:
            if ext.obj and hasattr(ext.obj, 'get_plugin_name'):
                names_dict[ext.name] = ext.obj.get_plugin_name()
        return names_dict

    ss_friendly_names = get_friendly_name_dict(secretstore_manager)
    crypto_friendly_names = get_friendly_name_dict(crypto_manager)
    # get existing secret stores data from database
    db_stores = secret_stores_repo.get_all()

    # read secret store data from service configuration
    conf_stores = []
    for parsed_store in secretstore_manager.parsed_stores:
        crypto_plugin = parsed_store.crypto_plugin
        if not crypto_plugin:
            crypto_plugin = None

        if crypto_plugin:
            friendly_name = crypto_friendly_names.get(crypto_plugin)
        else:
            friendly_name = ss_friendly_names.get(parsed_store.store_plugin)

        conf_stores.append(db_models.SecretStores(
            name=friendly_name, store_plugin=parsed_store.store_plugin,
            crypto_plugin=crypto_plugin,
            global_default=parsed_store.global_default))

    if db_stores:
        def fn_match(lh_store, rh_store):
            return (lh_store.store_plugin == rh_store.store_plugin and
                    lh_store.crypto_plugin == rh_store.crypto_plugin)

        for conf_store in conf_stores:
            # find existing db entry for plugin using conf based plugin names
            db_store_match = next((db_store for db_store in db_stores if
                                   fn_match(conf_store, db_store)), None)
            if db_store_match:
                # update existing db entry if global default is changed now
                if db_store_match.global_default != conf_store.global_default:
                    db_store_match.global_default = conf_store.global_default
                    # persist flag change.
                    db_store_match.save()
                # remove matches store from local list after processing
                db_stores.remove(db_store_match)
            else:  # new conf entry as no match found in existing entries
                secret_stores_repo.create_from(conf_store)

        # entries still present in db list are no longer configured in service
        # configuration, so try to remove them provided there is no project
        # is using it as preferred secret store.
        for db_store in db_stores:
            if proj_store_repo.get_count_by_secret_store(db_store.id) == 0:
                secret_stores_repo.delete_entity_by_id(db_store.id, None)
            else:
                raise exception.MultipleStorePluginStillInUse(db_store.name)
    else:  # initial setup case when there is no secret stores data in db
        for conf_store in conf_stores:
            secret_stores_repo.create_from(conf_store)


def get_global_default_secret_store():
    secret_store_repo = db_repos.get_secret_stores_repository()

    default_ss = None
    for secret_store in secret_store_repo.get_all():
        if secret_store.global_default:
            default_ss = secret_store
            break
    return default_ss


def get_applicable_crypto_plugins(manager, project_id, existing_plugin_name):
    """Get list of crypto plugins available for use.

    :param: manager instance of crypto manager
    :param: project_id project to identify preferred store if set
    :param: existing_plugin_name full plugin name. If a secret has an existing
            plugin defined, then we do not care if any preferred plugins have
            been defined. We will return all configured plugins as if multiple
            plugin support was not enabled. Subsequent code in the caller will
            select the plugin by name.

    When multiple backends support is enabled:
    It return project preferred plugin as list when it is setup earlier.
    If project preferred plugin is not set, then it uses plugin from default
    secret store.
    Plugin name is 'crypto_plugin' field value on identified secret store data.
    It returns matched plugin as list to match existing functionality.

    When multiple backends support is NOT enabled:
    In this case, it just returns list of all active plugins which is
    existing functionality before support for multiple backends is added.
    """
    return _get_applicable_plugins_for_type(manager, project_id,
                                            existing_plugin_name,
                                            'crypto_plugin')


def get_applicable_store_plugins(manager, project_id, existing_plugin_name):
    """Get list of secret store plugins available for use.

    :param: manager instance of secret store manager
    :param: project_id project to identify preferred store if set
    :param: existing_plugin_name full plugin name. If a secret has an existing
            plugin defined, then we do not care if any preferred plugins have
            been defined. We will return all configured plugins as if multiple
            plugin support was not enabled. Subsequent code in the caller will
            select the plugin by name.

    When multiple backends support is enabled:
    It return project preferred plugin as list when it is setup earlier.
    If project preferred plugin is not set, then it uses plugin from default
    secret store.
    Plugin name is 'store_plugin' field value on identified secret store data.
    It returns matched plugin as list to match existing functionality.

    When multiple backends support is NOT enabled:
    In this case, it just returns list of all active plugins which is
    existing functionality before support for multiple backends is added.
    """
    return _get_applicable_plugins_for_type(manager, project_id,
                                            existing_plugin_name,
                                            'store_plugin')


def _get_applicable_plugins_for_type(manager, project_id, existing_plugin_name,
                                     plugin_type_field):

    plugins = []
    plugin_dict = {ext.name: ext.obj for ext in manager.extensions if ext.obj}
    if utils.is_multiple_backends_enabled() and existing_plugin_name is None:
        proj_store_repo = db_repos.get_project_secret_store_repository()
        plugin_store = proj_store_repo.get_secret_store_for_project(
            project_id, None, suppress_exception=True)

        # If project specific store is not set, then use global default one.
        if not plugin_store:
            if manager.global_default_store_dict is None:
                # Need to cache data as dict instead of db object to be usable
                # across various request sqlalchemy sessions
                store_dict = get_global_default_secret_store().to_dict_fields()
                manager.global_default_store_dict = store_dict
            secret_store_data = manager.global_default_store_dict
        else:
            secret_store_data = plugin_store.secret_store.to_dict_fields()

        applicable_plugin_name = secret_store_data[plugin_type_field]
        if applicable_plugin_name in plugin_dict:
            plugins = [plugin_dict.get(applicable_plugin_name)]
        elif applicable_plugin_name:  # applicable_plugin_name has value
            raise exception.MultipleStorePreferredPluginMissing(
                applicable_plugin_name)
    else:
        plugins = plugin_dict.values()

    return plugins



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import itertools

from barbican.common.policies import acls
from barbican.common.policies import base
from barbican.common.policies import consumers
from barbican.common.policies import containers
from barbican.common.policies import orders
from barbican.common.policies import quotas
from barbican.common.policies import secretmeta
from barbican.common.policies import secrets
from barbican.common.policies import secretstores
from barbican.common.policies import transportkeys


def list_rules():
    return itertools.chain(
        base.list_rules(),
        acls.list_rules(),
        consumers.list_rules(),
        containers.list_rules(),
        orders.list_rules(),
        quotas.list_rules(),
        secretmeta.list_rules(),
        secrets.list_rules(),
        secretstores.list_rules(),
        transportkeys.list_rules(),
    )



"""
Copyright 2014-2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from functionaltests.api.v1.models import base_models


class SecretRefModel(base_models.BaseModel):

    def __init__(self, name=None, secret_ref=None):
        self.name = name
        self.secret_ref = secret_ref


class ContainerModel(base_models.BaseModel):

    def __init__(self, name=None, type=None, secret_refs=[],
                 container_ref=None, consumers=None, status=None,
                 updated=None, created=None, creator_id=None):
        self.name = name
        self.type = type
        self.secret_refs = secret_refs
        self.container_ref = container_ref
        self.consumers = consumers
        self.status = status
        self.updated = updated
        self.created = created
        self.creator_id = creator_id

    @classmethod
    def dict_to_obj(cls, input_dict):
        secret_refs = [SecretRefModel(**secret_ref) for secret_ref in
                       input_dict.get('secret_refs', [])]
        return cls(input_dict.get('name'), input_dict.get('type'), secret_refs,
                   container_ref=input_dict.get('container_ref'))



# Copyright (c) 2015 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from barbican.common import config
from barbican.common import exception
from barbican.common import hrefs
from barbican.common import resources as res
from barbican.model import repositories as repo


# All negative values will be treated as unlimited
UNLIMITED_VALUE = -1
DISABLED_VALUE = 0

CONF = config.CONF


class QuotaDriver(object):
    """Driver to enforce quotas and obtain quota information."""

    def __init__(self):
        self.repo = repo.get_project_quotas_repository()

    def _get_resources(self):
        """List of resources that can be constrained by a quota"""
        return ['secrets', 'orders', 'containers', 'consumers', 'cas']

    def _get_defaults(self):
        """Return list of default quotas"""
        quotas = {
            'secrets': CONF.quotas.quota_secrets,
            'orders': CONF.quotas.quota_orders,
            'containers': CONF.quotas.quota_containers,
            'consumers': CONF.quotas.quota_consumers,
            'cas': CONF.quotas.quota_cas
        }
        return quotas

    def _extract_project_quotas(self, project_quotas_model):
        """Convert project quotas model to Python dict

        :param project_quotas_model: Model containing quota information
        :return: Python dict containing quota information
        """
        resp_quotas = {}
        for resource in self._get_resources():
            resp_quotas[resource] = getattr(project_quotas_model, resource)
        return resp_quotas

    def _compute_effective_quotas(self, configured_quotas):
        """Merge configured and default quota information

        When a quota value is not set, use the default value
        :param configured_quotas: configured quota values
        :return: effective quotas
        """
        default_quotas = self._get_defaults()
        resp_quotas = dict(configured_quotas)
        for resource, quota in resp_quotas.items():
            if quota is None:
                resp_quotas[resource] = default_quotas[resource]
        return resp_quotas

    def get_effective_quotas(self, external_project_id):
        """Collect and return the effective quotas for a project

        :param external_project_id: external ID of current project
        :return: dict with effective quotas
        """
        try:
            retrieved_project_quotas = self.repo.get_by_external_project_id(
                external_project_id)
        except exception.NotFound:
            resp_quotas = self._get_defaults()
        else:
            resp_quotas = self._compute_effective_quotas(
                self._extract_project_quotas(retrieved_project_quotas))
        return resp_quotas

    def is_unlimited_value(self, v):
        """A helper method to check for unlimited value."""
        return v <= UNLIMITED_VALUE

    def is_disabled_value(self, v):
        """A helper method to check for disabled value."""
        return v == DISABLED_VALUE

    def set_project_quotas(self, external_project_id, parsed_project_quotas):
        """Create a new database entry, or update existing one

        :param external_project_id: ID of project whose quotas are to be set
        :param parsed_project_quotas: quota values to save in database
        :return: None
        """
        project = res.get_or_create_project(external_project_id)
        self.repo.create_or_update_by_project_id(project.id,
                                                 parsed_project_quotas)
        # commit to DB to avoid async issues if the enforcer is called from
        # another thread
        repo.commit()

    def get_project_quotas(self, external_project_id):
        """Retrieve configured quota information from database

        :param external_project_id: ID of project for whose values are wanted
        :return: the values
        """
        try:
            retrieved_project_quotas = self.repo.get_by_external_project_id(
                external_project_id)
        except exception.NotFound:
            return None
        resp_quotas = self._extract_project_quotas(retrieved_project_quotas)
        resp = {'project_quotas': resp_quotas}
        return resp

    def get_project_quotas_list(self, offset_arg=None, limit_arg=None):
        """Return a dict and list of all configured quota information

        :return: a dict and list of a page of quota config info
        """
        retrieved_project_quotas, offset, limit, total =\
            self.repo.get_by_create_date(offset_arg=offset_arg,
                                         limit_arg=limit_arg,
                                         suppress_exception=True)
        resp_quotas = []
        for quotas in retrieved_project_quotas:
            list_item = {'project_id': quotas.project.external_id,
                         'project_quotas':
                             self._extract_project_quotas(quotas)}
            resp_quotas.append(list_item)
        resp = {'project_quotas': resp_quotas}
        resp_overall = hrefs.add_nav_hrefs(
            'project_quotas', offset, limit, total, resp)
        resp_overall.update({'total': total})
        return resp_overall

    def delete_project_quotas(self, external_project_id):
        """Remove configured quota information from database

        :param external_project_id: ID of project whose quotas will be deleted
        :raises NotFound: if project has no configured values
        :return: None
        """
        self.repo.delete_by_external_project_id(external_project_id)

    def get_quotas(self, external_project_id):
        """Get the effective quotas for a project

        Effective quotas are based on both configured and default values
        :param external_project_id: ID of project for which to get quotas
        :return: dict of effective quota values
        """
        resp_quotas = self.get_effective_quotas(external_project_id)
        resp = {'quotas': resp_quotas}
        return resp


class QuotaEnforcer(object):
    """Checks quotas limits and current resource usage levels"""
    def __init__(self, resource_type, resource_repo):
        self.quota_driver = QuotaDriver()
        self.resource_type = resource_type
        self.resource_repo = resource_repo

    def enforce(self, project):
        """Enforce the quota limit for the resource

        :param project: the project object corresponding to the sender
        :raises QuotaReached: exception raised if quota forbids request
        :return: None
        """
        quotas = self.quota_driver.get_effective_quotas(project.external_id)
        quota = quotas[self.resource_type]

        reached = False
        count = 0
        if self.quota_driver.is_unlimited_value(quota):
            pass
        elif self.quota_driver.is_disabled_value(quota):
            reached = True
        else:
            count = self.resource_repo.get_count(project.id)
            if count >= quota:
                reached = True

        if reached:
            raise exception.QuotaReached(
                external_project_id=project.external_id,
                resource_type=self.resource_type,
                quota=quota)



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This test module focuses on RBAC interactions with the API resource classes.
For typical-flow business logic tests of these classes, see the
'resources_test.py' module.
"""
import os
from unittest import mock

from webob import exc

from barbican.api.controllers import consumers
from barbican.api.controllers import containers
from barbican.api.controllers import orders
from barbican.api.controllers import secrets
from barbican.api.controllers import secretstores
from barbican.api.controllers import versions
from barbican.common import accept as common_accept
from barbican.common import config
from barbican.common import policy
from barbican import context
from barbican.model import models
from barbican.tests import utils


# Point to the policy.json file located in source control.
TEST_VAR_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                            '../../../etc', 'barbican'))

CONF = config.new_config()

policy.init()
ENFORCER = policy.ENFORCER


class TestableResource(object):

    def __init__(self, *args, **kwargs):
        self.controller = self.controller_cls(*args, **kwargs)

    def on_get(self, req, resp, *args, **kwargs):
        with mock.patch('pecan.request', req):
            with mock.patch('pecan.response', resp):
                return self.controller.on_get(*args, **kwargs)

    def on_post(self, req, resp, *args, **kwargs):
        with mock.patch('pecan.request', req):
            with mock.patch('pecan.response', resp):
                return self.controller.on_post(*args, **kwargs)

    def on_put(self, req, resp, *args, **kwargs):
        with mock.patch('pecan.request', req):
            with mock.patch('pecan.response', resp):
                return self.controller.on_put(*args, **kwargs)

    def on_delete(self, req, resp, *args, **kwargs):
        with mock.patch('pecan.request', req):
            with mock.patch('pecan.response', resp):
                return self.controller.on_delete(*args, **kwargs)


class VersionsResource(TestableResource):
    controller_cls = versions.VersionsController


class SecretsResource(TestableResource):
    controller_cls = secrets.SecretsController


class SecretResource(TestableResource):
    controller_cls = secrets.SecretController


class OrdersResource(TestableResource):
    controller_cls = orders.OrdersController


class OrderResource(TestableResource):
    controller_cls = orders.OrderController


class ContainerResource(TestableResource):
    controller_cls = containers.ContainerController


class ConsumersResource(TestableResource):
    controller_cls = consumers.ContainerConsumersController


class ConsumerResource(TestableResource):
    controller_cls = consumers.ContainerConsumerController


class SecretStoresResource(TestableResource):
    controller_cls = secretstores.SecretStoresController


class SecretStoreResource(TestableResource):
    controller_cls = secretstores.SecretStoreController


class PreferredSecretStoreResource(TestableResource):
    controller_cls = secretstores.PreferredSecretStoreController


class SecretConsumersResource(TestableResource):
    controller_cls = consumers.SecretConsumersController


class SecretConsumerResource(TestableResource):
    controller_cls = consumers.SecretConsumerController


class BaseTestCase(utils.BaseTestCase, utils.MockModelRepositoryMixin):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        CONF(args=['--config-dir', TEST_VAR_DIR])
        self.policy_enforcer = ENFORCER
        self.policy_enforcer.load_rules(True)
        self.resp = mock.MagicMock()

    def _generate_req(self, roles=None, accept=None, content_type=None,
                      user_id=None, project_id=None):
        """Generate a fake HTTP request with security context added to it."""
        req = mock.MagicMock()
        req.get_param.return_value = None

        kwargs = {
            'user_id': user_id,
            'project_id': project_id,
            'roles': roles or [],
            'policy_enforcer': self.policy_enforcer,
        }
        req.environ = {}
        req.environ['barbican.context'] = context.RequestContext(**kwargs)
        req.content_type = content_type
        req.accept = common_accept.create_accept_header(accept)

        return req

    def _generate_stream_for_exit(self):
        """Mock HTTP stream generator, to force RBAC-pass exit.

        Generate a fake HTTP request stream that forces an IOError to
        occur, which short circuits API resource processing when RBAC
        checks under test here pass.
        """
        stream = mock.MagicMock()
        read = mock.MagicMock(return_value=None, side_effect=IOError())
        stream.read = read
        return stream

    def _assert_post_rbac_exception(self, exception, role):
        """Assert that we received the expected RBAC-passed exception."""
        self.assertEqual(500, exception.status_int)

    def _generate_get_error(self):
        """Falcon exception generator to throw from early-exit mocks.

        Creates an exception that should be raised by GET tests that pass
        RBAC. This allows such flows to short-circuit normal post-RBAC
        processing that is not tested in this module.

        :return: Python exception that should be raised by repo get methods.
        """
        # The 'Read Error' clause needs to match that asserted in
        #    _assert_post_rbac_exception() above.
        return exc.HTTPServerError(detail='Read Error')

    def _assert_pass_rbac(self, roles, method_under_test, accept=None,
                          content_type=None, user_id=None, project_id=None):
        """Assert that RBAC authorization rules passed for the specified roles.

        :param roles: List of roles to check, one at a time
        :param method_under_test: The test method to invoke for each role.
        :param accept Optional Accept header to set on the HTTP request
        :return: None
        """
        for role in roles:
            self.req = self._generate_req(roles=[role] if role else [],
                                          accept=accept,
                                          content_type=content_type,
                                          user_id=user_id,
                                          project_id=project_id)

            # Force an exception early past the RBAC passing.
            type(self.req).body = mock.PropertyMock(side_effect=IOError)
            self.req.body_file = self._generate_stream_for_exit()
            exception = self.assertRaises(exc.HTTPServerError,
                                          method_under_test)
            self._assert_post_rbac_exception(exception, role)

    def _assert_fail_rbac(self, roles, method_under_test, accept=None,
                          content_type=None, user_id=None, project_id=None):
        """Assert that RBAC rules failed for one of the specified roles.

        :param roles: List of roles to check, one at a time
        :param method_under_test: The test method to invoke for each role.
        :param accept Optional Accept header to set on the HTTP request
        :return: None
        """
        for role in roles:
            self.req = self._generate_req(roles=[role] if role else [],
                                          accept=accept,
                                          content_type=content_type,
                                          user_id=user_id,
                                          project_id=project_id)

            exception = self.assertRaises(exc.HTTPForbidden, method_under_test)
            self.assertEqual(403, exception.status_int)


class WhenTestingVersionsResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.VersionsResource class."""
    def setUp(self):
        super(WhenTestingVersionsResource, self).setUp()

        self.resource = VersionsResource()

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_versions(self):
        # Can't use base method that short circuits post-RBAC processing here,
        # as version GET is trivial
        for role in ['admin', 'observer', 'creator', 'audit']:
            self.req = self._generate_req(roles=[role] if role else [])
            self._invoke_on_get()

    def test_should_pass_get_versions_with_bad_roles(self):
        self.req = self._generate_req(roles=[None, 'bunkrolehere'])
        self._invoke_on_get()

    def test_should_pass_get_versions_with_no_roles(self):
        self.req = self._generate_req()
        self._invoke_on_get()

    def test_should_pass_get_versions_multiple_roles(self):
        self.req = self._generate_req(roles=['admin', 'observer', 'creator',
                                             'audit'])
        self._invoke_on_get()

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingSecretsResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.SecretsResource class."""
    def setUp(self):
        super(WhenTestingSecretsResource, self).setUp()

        self.external_project_id = '12345'

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.secret_repo = mock.MagicMock()
        get_by_create_date = mock.MagicMock(return_value=None,
                                            side_effect=self
                                            ._generate_get_error())
        self.secret_repo.get_by_create_date = get_by_create_date
        self.setup_secret_repository_mock(self.secret_repo)

        self.setup_encrypted_datum_repository_mock()
        self.setup_kek_datum_repository_mock()
        self.setup_project_repository_mock()
        self.setup_secret_meta_repository_mock()
        self.setup_transport_key_repository_mock()

        self.resource = SecretsResource()

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_create_secret(self):
        self._assert_pass_rbac(['admin', 'creator'], self._invoke_on_post,
                               content_type='application/json')

    def test_should_raise_create_secret(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'bogus'],
                               self._invoke_on_post,
                               content_type='application/json')

    def test_should_pass_get_secrets(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator'],
                               self._invoke_on_get,
                               content_type='application/json')

    def test_should_raise_get_secrets(self):
        self._assert_fail_rbac([None, 'audit', 'bogus'],
                               self._invoke_on_get,
                               content_type='application/json')

    def _invoke_on_post(self):
        self.resource.on_post(self.req, self.resp)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingSecretResource(BaseTestCase):
    """RBAC tests for SecretController class."""

    def setUp(self):
        super(WhenTestingSecretResource, self).setUp()

        self.external_project_id = '12345project'
        self.secret_id = '12345secret'
        self.user_id = '123456user'
        self.creator_user_id = '123456CreatorUser'

        # Force an error on GET and DELETE calls that pass RBAC,
        #   as we are not testing such flows in this test module.
        self.secret_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.secret_repo.get = fail_method
        self.secret_repo.delete_entity_by_id = fail_method
        self.setup_secret_repository_mock(self.secret_repo)

        self.setup_encrypted_datum_repository_mock()
        self.setup_kek_datum_repository_mock()
        self.setup_project_repository_mock()
        self.setup_secret_meta_repository_mock()
        self.setup_transport_key_repository_mock()

        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=True,
                                    user_ids=[self.user_id, 'anyRandomId'])
        self.acl_list = [acl_read]
        secret = mock.MagicMock()
        secret.secret_acls.__iter__.return_value = self.acl_list
        secret.project.external_id = self.external_project_id
        secret.creator_id = self.creator_user_id

        self.resource = SecretResource(secret)

        # self.resource.controller.get_acl_tuple = mock.MagicMock(
        #    return_value=(None, None))

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_decrypt_secret(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_decrypt_secret(self):

        self._assert_fail_rbac([None, 'audit', 'bogus'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype')

    def test_should_pass_decrypt_secret_for_same_project_with_no_acl(self):
        """Token and secret project needs to be same in no ACL defined case."""
        self.acl_list.pop()  # remove read acl from default setup
        self._assert_pass_rbac(['admin', 'observer', 'creator'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_decrypt_secret_with_project_access_disabled(self):
        """Should raise authz error as secret is marked private.

        As secret is private so project users should not be able to access
        the secret. Admin project user can still access it.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_fail_rbac(['observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_pass_decrypt_secret_for_admin_user_project_access_disabled(self):
        """Should pass authz for admin role user as secret is marked private.

        Even when secret is private, admin user should still have access to
        the secret.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_decrypt_secret_for_with_project_access_nolist(self):
        """Should raise authz error as secret is marked private.

        As secret is private so project users should not be able to access
        the secret.  This test passes user_ids as empty list, which is a
        valid and common case. Admin project user can still access it.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=[])
        self.acl_list.append(acl_read)
        self._assert_fail_rbac(['observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_decrypt_secret_private_enabled_with_read_acl(self):
        """Should pass authz as user has read acl for private secret.

        Even though secret is private, user with read acl should be able to
        access the secret.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id='aclUser1',
                               project_id=self.external_project_id)

    def test_should_pass_decrypt_secret_different_user_valid_read_acl(self):
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=True,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        # token project_id is different from secret's project id but another
        # user (from different project) has read acl for secret so should pass
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id='aclUser1',
                               project_id='different_project_id')

    def test_should_raise_decrypt_secret_for_different_user_no_read_acl(self):
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id,
                                    operation='write',
                                    project_access=True,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        # token project_id is different from secret's project id but another
        # user (from different project) has read acl for secret so should pass
        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id='aclUser1',
                               project_id='different_project_id')

    def test_fail_decrypt_secret_for_creator_user_with_different_project(self):
        """Check for creator user rule for secret decrypt/get call.

        If token's user is creator of secret but its scoped to different
        project, then he/she is not allowed access to secret when project
        is marked private.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id,
                                    operation='write',
                                    project_access=True,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self.resource.controller.secret.creator_id = 'creatorUserX'
        # token user is creator but scoped to project different from secret
        # project so don't allow decrypt secret call to creator of that secret
        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               accept='notjsonaccepttype',
                               content_type='application/json',
                               user_id='creatorUserX',
                               project_id='different_project_id')

    def test_should_pass_get_secret(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_get_secret_with_no_context(self):
        """In unauthenticated flow, get secret should work."""
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get_without_context,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_get_secret_for_different_project_no_acl(self):
        """Should raise error when secret and token's project is different."""
        self.acl_list.pop()  # remove read acl from default setup
        # token project_id is different from secret's project id so should fail
        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id='different_id')

    def test_should_pass_get_secret_for_same_project_but_different_user(self):
        # user id should not matter as long token and secret's project match
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id='different_user_id',
                               project_id=self.external_project_id)

    def test_should_pass_get_secret_for_same_project_with_no_acl(self):
        self.acl_list.pop()  # remove read acl from default setup
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_get_secret_for_with_project_access_disabled(self):
        """Should raise authz error as secret is marked private.

        As secret is private so project users should not be able to access
        the secret.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_fail_rbac(['observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_pass_get_secret_for_admin_user_with_project_access_disabled(self):
        """Should pass authz for admin user as secret is marked private.

        Even when secret is private, admin user should have access
        the secret.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_get_secret_for_private_enabled_with_read_acl(self):
        """Should pass authz as user has read acl for private secret.

        Even though secret is private, user with read acl should be able to
        access the secret.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=False,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               user_id='aclUser1',
                               project_id=self.external_project_id)

    def test_should_pass_get_secret_different_user_with_valid_read_acl(self):
        """Should allow when read ACL is defined for a user.

        Secret's own project and token's project is different but read is
        allowed because of valid read ACL.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id, operation='read',
                                    project_access=True,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        # token project_id is different from secret's project id but another
        # user (from different project) has read acl for secret so should pass
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               user_id='aclUser1',
                               project_id='different_project_id')

    def test_should_raise_get_secret_for_different_user_with_no_read_acl(self):
        """Get secret fails when no read acl is defined.

        With different secret and token's project, read is not allowed without
        a read ACL.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.SecretACL(secret_id=self.secret_id,
                                    operation='write',
                                    project_access=True,
                                    user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        # token project_id is different from secret's project id but another
        # user (from different project) has read acl for secret so should pass
        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id='aclUser1',
                               project_id='different_project_id')

    def test_fail_get_secret_for_creator_user_with_different_project(self):
        """Check for creator user rule for secret get call.

        If token's user is creator of secret but its scoped to different
        project, then he/she is not allowed access to secret when project
        is marked private.
        """
        self.acl_list.pop()  # remove read acl from default setup
        self.resource.controller.secret.creator_id = 'creatorUserX'

        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               user_id='creatorUserX',
                               project_id='different_project_id')

    def test_should_raise_get_secret(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get)

    def test_should_pass_put_secret(self):
        self._assert_pass_rbac(['admin', 'creator'], self._invoke_on_put,
                               content_type="application/octet-stream",
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_put_secret(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'bogus'],
                               self._invoke_on_put,
                               content_type="application/octet-stream")

    def test_should_pass_delete_secret(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_delete,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_delete_secret(self):
        """A non-admin user cannot delete other user's secret.

        User id is different from initial user who has created the secret.
        """
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_delete,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_delete_secret_for_owner(self):
        """Non-admin user can delete his/her own secret

        Secret creator_id should match with token user to establish ownership.
        """
        self._assert_pass_rbac(['creator'], self._invoke_on_delete,
                               user_id=self.creator_user_id,
                               project_id=self.external_project_id)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)

    def _invoke_on_get_without_context(self):
        # Adding this to get code coverage around context check lines
        self.req.environ.pop('barbican.context')
        self.resource.on_get(self.req, self.resp,
                             self.external_project_id)

    def _invoke_on_put(self):
        self.resource.on_put(self.req, self.resp)

    def _invoke_on_delete(self):
        self.resource.on_delete(self.req, self.resp)


class WhenTestingContainerResource(BaseTestCase):
    """RBAC tests for ContainerController class.

    Container controller tests are quite similar to SecretController as
    policy logic is same. Just adding them here to make sure logic related to
    acl gathering data works as expected.
    """

    def setUp(self):
        super(WhenTestingContainerResource, self).setUp()

        self.external_project_id = '12345project'
        self.container_id = '12345secret'
        self.user_id = '123456user'
        self.creator_user_id = '123456CreatorUser'

        # Force an error on GET and DELETE calls that pass RBAC,
        #   as we are not testing such flows in this test module.
        self.container_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.container_repo.get = fail_method
        self.container_repo.delete_entity_by_id = fail_method

        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=True, user_ids=[self.user_id, 'anyRandomId'])
        self.acl_list = [acl_read]
        container = mock.MagicMock()
        container.to_dict_fields = mock.MagicMock(side_effect=IOError)
        container.id = self.container_id
        container.container_acls.__iter__.return_value = self.acl_list
        container.project.external_id = self.external_project_id
        container.creator_id = self.creator_user_id

        self.container_repo.get_container_by_id.return_value = container

        self.setup_container_repository_mock(self.container_repo)

        self.resource = ContainerResource(container)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_container(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_get_container_with_no_context(self):
        """In unauthenticated flow, get container should work."""
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get_without_context,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_get_container_for_different_project_no_acl(self):
        """Raise error when container and token's project is different."""
        self.acl_list.pop()  # remove read acl from default setup
        # token project_id is different from secret's project id so should fail
        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id='different_id')

    def test_should_pass_get_container_for_same_project_but_different_user(
            self):
        """Should pass if token and secret's project match.

        User id should not matter as long token and container's project match.
        """
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id='different_user_id',
                               project_id=self.external_project_id)

    def test_should_pass_get_container_for_same_project_with_no_acl(self):
        self.acl_list.pop()  # remove read acl from default setup
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_get_container_for_with_project_access_disabled(self):
        """Should raise authz error as container is marked private.

        As container is private so project users should not be able to access
        the secret (other than admin user).
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=False, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_fail_rbac(['observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_pass_get_container_for_admin_user_project_access_disabled(self):
        """Should pass authz for admin user when container is marked private.

        For private container, admin user should still be able to access
        the secret.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=False, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin'],
                               self._invoke_on_get,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_get_container_for_private_enabled_with_read_acl(self):
        """Should pass authz as user has read acl for private container.

        Even though container is private, user with read acl should be able to
        access the container.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=False, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               user_id='aclUser1',
                               project_id=self.external_project_id)

    def test_should_pass_get_container_different_user_with_valid_read_acl(
            self):
        """Should allow when read ACL is defined for a user.

        Container's own project and token's project is different but read is
        allowed because of valid read ACL. User can read regardless of what is
        token's project as it has necessary ACL.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=True, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit',
                                'bogusRole'],
                               self._invoke_on_get,
                               user_id='aclUser1',
                               project_id='different_project_id')

    def test_should_raise_get_container_for_different_user_with_no_read_acl(
            self):
        """Get secret fails when no read acl is defined.

        With different container and token's project, read is not allowed
        without a read ACL.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='write',
            project_access=True, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        # token project_id is different from secret's project id but another
        # user (from different project) has read acl for secret so should pass
        self._assert_fail_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               user_id='aclUser1',
                               project_id='different_project_id')

    def test_fail_get_container_for_creator_user_different_project(self):
        """Check for creator user rule for container get call.

        If token's user is creator of container but its scoped to different
        project, then he/she is not allowed access to container when project
        is marked private.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=False, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_fail_rbac(['creator'],
                               self._invoke_on_get,
                               user_id=self.creator_user_id,
                               project_id='differet_project_id')

    def test_pass_get_container_for_creator_user_project_access_disabled(self):
        """Should pass authz for creator user when container is marked private.

        As container is private so user who created the container can still
        access it as long as user has 'creator' role in container project.
        """
        self.acl_list.pop()  # remove read acl from default setup
        acl_read = models.ContainerACL(
            container_id=self.container_id, operation='read',
            project_access=False, user_ids=['anyRandomUserX', 'aclUser1'])
        self.acl_list.append(acl_read)
        self._assert_pass_rbac(['creator'],
                               self._invoke_on_get,
                               user_id=self.creator_user_id,
                               project_id=self.external_project_id)

    def test_should_raise_get_container(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get)

    def test_should_pass_delete_container(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_delete,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_raise_delete_container(self):
        """A non-admin user cannot delete other user's container.

        User id is different from initial user who has created the container.
        """
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_delete,
                               user_id=self.user_id,
                               project_id=self.external_project_id)

    def test_should_pass_delete_container_for_owner(self):
        """Non-admin user can delete his/her own container

        Container creator_id should match with token user to establish
        ownership.
        """
        self._assert_pass_rbac(['creator'], self._invoke_on_delete,
                               user_id=self.creator_user_id,
                               project_id=self.external_project_id)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)

    def _invoke_on_get_without_context(self):
        # Adding this to get code coverage around context check lines
        self.req.environ.pop('barbican.context')
        self.resource.on_get(self.req, self.resp)

    def _invoke_on_put(self):
        self.resource.on_put(self.req, self.resp)

    def _invoke_on_delete(self):
        self.resource.on_delete(self.req, self.resp)


class WhenTestingOrdersResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.OrdersResource class."""
    def setUp(self):
        super(WhenTestingOrdersResource, self).setUp()

        self.external_project_id = '12345'

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.order_repo = mock.MagicMock()
        get_by_create_date = mock.MagicMock(return_value=None,
                                            side_effect=self
                                            ._generate_get_error())
        self.order_repo.get_by_create_date = get_by_create_date

        self.setup_order_repository_mock(self.order_repo)
        self.setup_project_repository_mock()

        self.resource = OrdersResource(queue_resource=mock.MagicMock())

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_create_order(self):
        self._assert_pass_rbac(['admin', 'creator'], self._invoke_on_post,
                               content_type='application/json')

    def test_should_raise_create_order(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'bogus'],
                               self._invoke_on_post)

    def test_should_pass_get_orders(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator'],
                               self._invoke_on_get)

    def test_should_raise_get_orders(self):
        self._assert_fail_rbac([None, 'audit', 'bogus'],
                               self._invoke_on_get)

    def _invoke_on_post(self):
        self.resource.on_post(self.req, self.resp)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingOrderResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.OrderResource class."""
    def setUp(self):
        super(WhenTestingOrderResource, self).setUp()

        self.external_project_id = '12345project'
        self.order_id = '12345order'

        # Force an error on GET and DELETE calls that pass RBAC,
        #   as we are not testing such flows in this test module.
        self.order_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.order_repo.get = fail_method
        self.order_repo.delete_entity_by_id = fail_method

        self.setup_order_repository_mock(self.order_repo)

        self.resource = OrderResource(self.order_id)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_order(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get)

    def test_should_raise_get_order(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get)

    def test_should_pass_delete_order(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_delete)

    def test_should_raise_delete_order(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_delete)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)

    def _invoke_on_delete(self):
        self.resource.on_delete(self.req, self.resp)


class WhenTestingConsumersResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.ConsumersResource class."""
    def setUp(self):
        super(WhenTestingConsumersResource, self).setUp()

        self.external_project_id = '12345project'
        self.container_id = '12345container'

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.consumer_repo = mock.MagicMock()
        get_by_container_id = mock.MagicMock(return_value=None,
                                             side_effect=self
                                             ._generate_get_error())
        self.consumer_repo.get_by_container_id = get_by_container_id

        self.setup_project_repository_mock()
        self.setup_container_consumer_repository_mock(self.consumer_repo)
        self.setup_container_repository_mock()

        self.resource = ConsumersResource(container_id=self.container_id)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_create_consumer(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_post,
                               content_type='application/json')

    def test_should_raise_create_consumer(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_post,
                               content_type='application/json')

    def test_should_pass_delete_consumer(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_delete,
                               content_type='application/json')

    def test_should_raise_delete_consumer(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_delete)

    def test_should_pass_get_consumers(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               content_type='application/json')

    def test_should_raise_get_consumers(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get,
                               content_type='application/json')

    def _invoke_on_post(self):
        self.resource.on_post(self.req, self.resp)

    def _invoke_on_delete(self):
        self.resource.on_delete(self.req, self.resp)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingConsumerResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.ConsumerResource class."""
    def setUp(self):
        super(WhenTestingConsumerResource, self).setUp()

        self.external_project_id = '12345project'
        self.consumer_id = '12345consumer'

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.consumer_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.consumer_repo.get = fail_method

        self.setup_project_repository_mock()
        self.setup_container_consumer_repository_mock(self.consumer_repo)
        self.resource = ConsumerResource(consumer_id=self.consumer_id)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_consumer(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get)

    def test_should_raise_get_consumer(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingSecretStoresResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.SecretStoresResource class."""
    def setUp(self):
        super(WhenTestingSecretStoresResource, self).setUp()

        self.external_project_id = '12345project'

        self.moc_enable_patcher = mock.patch(
            'barbican.common.utils.is_multiple_backends_enabled')
        enable_check_method = self.moc_enable_patcher.start()
        enable_check_method.return_value = True
        self.addCleanup(self.moc_enable_patcher.stop)

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.project_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.project_repo.find_by_external_project_id = fail_method
        self.setup_project_repository_mock(self.project_repo)

        self.resource = SecretStoresResource()

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_all_secret_stores(self):
        self._assert_pass_rbac(['admin'],
                               self._invoke_on_get)

    def test_should_raise_get_all_secret_stores(self):
        self._assert_fail_rbac([None, 'creator', 'observer', 'audit'],
                               self._invoke_on_get)

    def test_should_pass_get_global_default(self):
        self._assert_pass_rbac(['admin'],
                               self._invoke_get_global_default)

    def test_should_raise_get_global_default(self):
        self._assert_fail_rbac([None, 'creator', 'observer', 'audit'],
                               self._invoke_get_global_default)

    def test_should_pass_get_preferred(self):
        self._assert_pass_rbac(['admin'],
                               self._invoke_get_preferred)

    def test_should_raise_get_preferred(self):
        self._assert_fail_rbac([None, 'creator', 'observer', 'audit'],
                               self._invoke_get_preferred)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)

    def _invoke_get_global_default(self):
        with mock.patch('pecan.request', self.req):
            with mock.patch('pecan.response', self.resp):
                return self.resource.controller.get_global_default()

    def _invoke_get_preferred(self):
        with mock.patch('pecan.request', self.req):
            with mock.patch('pecan.response', self.resp):
                return self.resource.controller.get_preferred()


class WhenTestingSecretStoreResource(BaseTestCase):
    """RBAC tests for the barbican.api.resources.SecretStoreResource class."""
    def setUp(self):
        super(WhenTestingSecretStoreResource, self).setUp()

        self.external_project_id = '12345project'
        self.store_id = '123456SecretStoreId'

        self.moc_enable_patcher = mock.patch(
            'barbican.common.utils.is_multiple_backends_enabled')
        enable_check_method = self.moc_enable_patcher.start()
        enable_check_method.return_value = True
        self.addCleanup(self.moc_enable_patcher.stop)

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.

        self.project_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())

        self.project_repo.find_by_external_project_id = fail_method
        self.setup_project_repository_mock(self.project_repo)

        secret_store_res = mock.MagicMock()
        secret_store_res.to_dict_fields = mock.MagicMock(side_effect=IOError)
        secret_store_res.id = self.store_id

        self.resource = SecretStoreResource(secret_store_res)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_a_secret_store(self):
        self._assert_pass_rbac(['admin'],
                               self._invoke_on_get)

    def test_should_raise_get_a_secret_store(self):
        self._assert_fail_rbac([None, 'creator', 'observer', 'audit'],
                               self._invoke_on_get)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingPreferredSecretStoreResource(BaseTestCase):
    """RBAC tests for barbican.api.resources.PreferredSecretStoreResource"""
    def setUp(self):
        super(WhenTestingPreferredSecretStoreResource, self).setUp()

        self.external_project_id = '12345project'
        self.store_id = '123456SecretStoreId'

        self.moc_enable_patcher = mock.patch(
            'barbican.common.utils.is_multiple_backends_enabled')
        enable_check_method = self.moc_enable_patcher.start()
        enable_check_method.return_value = True
        self.addCleanup(self.moc_enable_patcher.stop)

        # Force an error on POST/DELETE calls that pass RBAC, as we are not
        # testing such flows in this test module.
        self.project_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.project_repo.find_by_external_project_id = fail_method
        self.setup_project_repository_mock(self.project_repo)

        self.resource = PreferredSecretStoreResource(mock.MagicMock())

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_set_preferred_secret_store(self):
        self._assert_pass_rbac(['admin'],
                               self._invoke_on_post)

    def test_should_raise_set_preferred_secret_store(self):
        self._assert_fail_rbac([None, 'creator', 'observer', 'audit'],
                               self._invoke_on_post)

    def _invoke_on_post(self):
        self.resource.on_post(self.req, self.resp)


class WhenTestingSecretConsumersResource(BaseTestCase):
    """RBAC tests for barbican.api.resources.SecretConsumersResource"""
    def setUp(self):
        super(WhenTestingSecretConsumersResource, self).setUp()

        self.external_project_id = '12345project'
        self.secret_id = '12345secret'

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.consumer_repo = mock.MagicMock()
        get_by_secret_id = mock.MagicMock(return_value=None,
                                          side_effect=self
                                          ._generate_get_error())
        self.consumer_repo.get_by_secret_id = get_by_secret_id

        self.setup_project_repository_mock()
        self.setup_secret_consumer_repository_mock(self.consumer_repo)
        self.setup_secret_repository_mock()

        self.resource = SecretConsumersResource(secret_id=self.secret_id)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_create_consumer(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_post,
                               content_type='application/json')

    def test_should_raise_create_consumer(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_post,
                               content_type='application/json')

    def test_should_pass_delete_consumer(self):
        self._assert_pass_rbac(['admin'], self._invoke_on_delete,
                               content_type='application/json')

    def test_should_raise_delete_consumer(self):
        self._assert_fail_rbac([None, 'audit', 'observer', 'creator', 'bogus'],
                               self._invoke_on_delete)

    def test_should_pass_get_consumers(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get,
                               content_type='application/json')

    def test_should_raise_get_consumers(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get,
                               content_type='application/json')

    def _invoke_on_post(self):
        self.resource.on_post(self.req, self.resp)

    def _invoke_on_delete(self):
        self.resource.on_delete(self.req, self.resp)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)


class WhenTestingSecretConsumerResource(BaseTestCase):
    """RBAC tests for barbican.api.resources.SecretConsumerResource"""
    def setUp(self):
        super(WhenTestingSecretConsumerResource, self).setUp()

        self.external_project_id = '12345project'
        self.consumer_id = '12345consumer'

        # Force an error on GET calls that pass RBAC, as we are not testing
        #   such flows in this test module.
        self.consumer_repo = mock.MagicMock()
        fail_method = mock.MagicMock(return_value=None,
                                     side_effect=self._generate_get_error())
        self.consumer_repo.get = fail_method

        self.setup_project_repository_mock()
        self.setup_secret_consumer_repository_mock(self.consumer_repo)
        self.resource = SecretConsumerResource(consumer_id=self.consumer_id)

    def test_rules_should_be_loaded(self):
        self.assertIsNotNone(self.policy_enforcer.rules)

    def test_should_pass_get_consumer(self):
        self._assert_pass_rbac(['admin', 'observer', 'creator', 'audit'],
                               self._invoke_on_get)

    def test_should_raise_get_consumer(self):
        self._assert_fail_rbac([None, 'bogus'],
                               self._invoke_on_get)

    def _invoke_on_get(self):
        self.resource.on_get(self.req, self.resp)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys
from unittest import mock

from barbican.cmd import retry_scheduler
from barbican.cmd import worker
from barbican.tests import database_utils
from barbican.tests.queue import test_keystone_listener
from barbican.tests import utils


class WhenInvokingRetryServiceCommand(utils.BaseTestCase):
    """Test the retry scheduler functionality."""

    def setUp(self):
        super(WhenInvokingRetryServiceCommand, self).setUp()
        sys.argv = ['barbican-retry']

    @mock.patch('barbican.common.config')
    @mock.patch('barbican.queue.init')
    @mock.patch('oslo_service.service.launch')
    @mock.patch('barbican.queue.retry_scheduler.PeriodicServer')
    def test_should_launch_service(
            self,
            mock_periodic_server,
            mock_service_launch,
            mock_queue_init,
            mock_config):

        retry_scheduler.main()

        self.assertEqual(1, mock_queue_init.call_count)
        self.assertEqual(1, mock_service_launch.call_count)
        self.assertEqual(1, mock_periodic_server.call_count)

    @mock.patch('oslo_log.log.setup')
    @mock.patch('sys.exit')
    def test_should_fail_run_command(
            self, mock_sys_exit, mock_log_setup):
        mock_log_setup.side_effect = RuntimeError()

        retry_scheduler.main()

        self.assertEqual(1, mock_sys_exit.call_count)


class WhenInvokingWorkerCommand(test_keystone_listener.UtilMixin,
                                utils.BaseTestCase):
    """Test the asynchronous worker functionality."""

    def setUp(self):
        super(WhenInvokingWorkerCommand, self).setUp()
        database_utils.setup_in_memory_db()
        sys.argv = ['barbican-worker']

    @mock.patch('barbican.queue.init')
    @mock.patch('barbican.queue.get_server')
    @mock.patch('oslo_service.service.launch')
    def test_should_launch_service(
            self,
            mock_service_launch,
            mock_queue_task_server,
            mock_queue_init):

        self.opt_in_group('queue', asynchronous_workers=3)
        worker.main()

        self.assertEqual(1, mock_queue_init.call_count)
        self.assertEqual(1, mock_service_launch.call_count)
        # check keyword argument for number of worker matches
        workers_kwarg = {'restart_method': 'mutate', 'workers': 3}
        self.assertEqual(workers_kwarg, mock_service_launch.call_args[1])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from alembic import script as alembic_script
from oslo_config import cfg
import six
import sqlalchemy

from barbican.common import config
from barbican.common import exception
from barbican.model.migration import commands as migration
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils
from barbican.tests import utils


class WhenCleaningRepositoryPagingParameters(utils.BaseTestCase):

    def setUp(self):
        super(WhenCleaningRepositoryPagingParameters, self).setUp()
        self.CONF = config.CONF
        self.default_limit = self.CONF.default_limit_paging

    def test_parameters_not_assigned(self):
        """The cleaner should use defaults when params are not specified."""
        clean_offset, clean_limit = repositories.clean_paging_values()

        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_limit_as_none(self):
        """When Limit is set to None it should use the default limit."""
        offset = 0
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=offset,
            limit_arg=None)

        self.assertEqual(offset, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_offset_as_none(self):
        """When Offset is set to None it should use an offset of 0."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=None,
            limit_arg=self.default_limit)

        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_limit_as_uncastable_str(self):
        """When Limit cannot be cast to an int, expect the default."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=0,
            limit_arg='boom')
        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_offset_as_uncastable_str(self):
        """When Offset cannot be cast to an int, it should be zero."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg='boom',
            limit_arg=self.default_limit)
        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_limit_is_less_than_one(self):
        """Offset should default to 1."""
        limit = -1
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=1,
            limit_arg=limit)
        self.assertEqual(1, clean_offset)
        self.assertEqual(1, clean_limit)

    def test_limit_is_too_big(self):
        """Limit should max out at configured value."""
        limit = self.CONF.max_limit_paging + 10
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=1,
            limit_arg=limit)
        self.assertEqual(self.CONF.max_limit_paging, clean_limit)

    def test_offset_is_too_big(self):
        """When Offset exceeds sys.maxsize, it should be zero."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=265613988875874769338781322035779626829233452653394495,
            limit_arg=self.default_limit)
        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)


class WhenInvokingExceptionMethods(utils.BaseTestCase):

    def setUp(self):
        super(WhenInvokingExceptionMethods, self).setUp()
        self.CONF = config.CONF

        self.entity_id = '123456'
        self.entity_name = 'test_entity'

    def test_should_raise_for_entity_not_found(self):

        exception_result = self.assertRaises(
            exception.NotFound,
            repositories._raise_entity_not_found,
            self.entity_name,
            self.entity_id)

        self.assertEqual(
            "No test_entity found with ID 123456",
            six.text_type(exception_result))

    def test_should_raise_for_entity_id_not_found(self):

        exception_result = self.assertRaises(
            exception.NotFound,
            repositories._raise_entity_id_not_found,
            self.entity_id)

        self.assertEqual(
            "Entity ID 123456 not found",
            six.text_type(exception_result))

    def test_should_raise_for_no_entities_found(self):

        exception_result = self.assertRaises(
            exception.NotFound,
            repositories._raise_no_entities_found,
            self.entity_name)

        self.assertEqual(
            "No entities of type test_entity found",
            six.text_type(exception_result))


class WhenTestingBaseRepository(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingBaseRepository, self).setUp()
        self.repo = repositories.BaseRepo()

    def test_should_raise_invalid_create_from_no_entity(self):
        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo.create_from,
            None)

        self.assertEqual(
            "Must supply non-None Entity.",
            six.text_type(exception_result))

    def test_should_raise_invalid_create_from_entity_with_id(self):
        entity = models.ModelBase()
        entity.id = '1234'

        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo.create_from,
            entity)

        self.assertEqual(
            "Must supply Entity with id=None (i.e. new entity).",
            six.text_type(exception_result))

    def test_should_raise_invalid_do_validate_no_status(self):
        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo._do_validate,
            {})

        self.assertEqual(
            "Entity status is required.",
            six.text_type(exception_result))

    def test_should_raise_invalid_do_validate_bad_status(self):
        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo._do_validate,
            dict(status='BOGUS_STATUS'))

        self.assertEqual(
            "Invalid status 'BOGUS_STATUS' for Entity.",
            six.text_type(exception_result))


class WhenTestingWrapDbError(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingWrapDbError, self).setUp()
        repositories.CONF.set_override("sql_max_retries", 0)
        repositories.CONF.set_override("sql_retry_interval", 0)

    @mock.patch('barbican.model.repositories.is_db_connection_error')
    def test_should_raise_operational_error_is_connection_error(
            self, mock_is_db_error):
        mock_is_db_error.return_value = True

        @repositories.wrap_db_error
        def test_function():
            raise sqlalchemy.exc.OperationalError(
                'statement', 'params', 'orig')

        self.assertRaises(
            sqlalchemy.exc.OperationalError,
            test_function)


class WhenTestingGetEnginePrivate(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingGetEnginePrivate, self).setUp()

        repositories.CONF.set_override("sql_connection", "connection")

    @mock.patch('barbican.model.repositories._create_engine')
    def test_should_raise_value_exception_engine_create_failure(
            self, mock_create_engine):
        engine = mock.MagicMock()
        engine.connect.side_effect = ValueError('Abort!')
        mock_create_engine.return_value = engine

        exception_result = self.assertRaises(
            exception.BarbicanException,
            repositories._get_engine,
            None)

        self.assertEqual(
            'Error configuring registry database with supplied '
            'sql_connection. Got error: Abort!',
            six.text_type(exception_result))

    @mock.patch('barbican.model.repositories._create_engine')
    def test_should_complete_with_no_alembic_create_default_configs(
            self, mock_create_engine):

        repositories.CONF.set_override("db_auto_create", False)
        engine = mock.MagicMock()
        mock_create_engine.return_value = engine

        # Invoke method under test.
        repositories._get_engine(None)

        engine.connect.assert_called_once_with()
        mock_create_engine.assert_called_once_with(
            'connection',
            idle_timeout=3600,
            max_pool_size=repositories.CONF.sql_pool_size,
            max_overflow=repositories.CONF.sql_pool_max_overflow
        )

    @mock.patch('barbican.model.repositories._create_engine')
    def test_should_complete_with_no_alembic_create_pool_configs(
            self, mock_create_engine):

        repositories.CONF.set_override("db_auto_create", False)
        repositories.CONF.set_override(
            "sql_pool_class", "QueuePool")
        repositories.CONF.set_override("sql_pool_size", 22)
        repositories.CONF.set_override("sql_pool_max_overflow", 11)

        engine = mock.MagicMock()
        mock_create_engine.return_value = engine

        # Invoke method under test.
        repositories._get_engine(None)

        engine.connect.assert_called_once_with()
        mock_create_engine.assert_called_once_with(
            'connection',
            idle_timeout=3600,
            max_pool_size=22,
            max_overflow=11
        )


class WhenTestingAutoGenerateTables(utils.BaseTestCase):

    @mock.patch('barbican.model.migration.commands.upgrade')
    def test_should_complete_with_alembic_database_update(
            self, mock_commands_upgrade):

        tables = dict(
            alembic_version='version')  # Mimic tables already created.
        engine = 'engine'

        # Invoke method under test.
        repositories._auto_generate_tables(engine, tables)

        mock_commands_upgrade.assert_called_once_with()


class WhenTestingIsDbConnectionError(utils.BaseTestCase):

    def test_should_return_false_no_error_code_in_args(self):

        args = mock.MagicMock()
        args.find.return_value = -1

        result = repositories.is_db_connection_error(args)

        self.assertFalse(result)

    def test_should_return_true_error_code_found_in_args(self):

        args = mock.MagicMock()
        args.find.return_value = 1

        result = repositories.is_db_connection_error(args)

        self.assertTrue(result)


class WhenTestingMigrations(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingMigrations, self).setUp()
        repositories.CONF.set_override("sql_connection", "connection")
        self.alembic_config = migration.init_config()
        self.alembic_config.barbican_config = cfg.CONF

    def test_no_downgrade(self):
        script_dir = alembic_script.ScriptDirectory.from_config(
            self.alembic_config)
        versions = [v for v in script_dir.walk_revisions(base='base',
                                                         head='heads')]
        failed_revisions = []
        for version in versions:
            if hasattr(version.module, 'downgrade'):
                failed_revisions.append(version.revision)

        if failed_revisions:
            self.fail('Migrations %s have downgrade' % failed_revisions)


class DummyRepo(repositories.BaseRepo):
    """Repository for the increasing code coverage of unit tests."""
    def get_session(self, session=None):
        return None

    def _do_entity_name(self):
        return "Dummy"

    def _do_build_get_query(self, entity_id, external_project_id, session):
        return None

    def _do_validate(self, values):
        pass

    def _build_get_project_entities_query(self, project_id, session):
        return None


class WhenIncreasingRepositoryTestCoverage(utils.BaseTestCase):

    def test_get_count_should_return_zero(self):
        dummy_repo = DummyRepo()
        count = dummy_repo.get_count('dummy_project_id')
        self.assertEqual(0, count)

    def test_get_project_entities_should_return_empty(self):
        dummy_repo = DummyRepo()
        count = dummy_repo.get_project_entities('dummy_project_id')
        self.assertEqual([], count)



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import mock

from cryptography import fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
import six

from barbican.model import models
from barbican.plugin.crypto import base as plugin
from barbican.plugin.crypto import simple_crypto as simple
from barbican.tests import utils


class WhenTestingSimpleCryptoPlugin(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingSimpleCryptoPlugin, self).setUp()
        self.plugin = simple.SimpleCryptoPlugin()

    def _get_mocked_kek_meta_dto(self):
        # For SimpleCryptoPlugin, per-project KEKs are stored in
        # kek_meta_dto.plugin_meta. SimpleCryptoPlugin does a get-or-create
        # on the plugin_meta field, so plugin_meta should be None initially.
        kek_meta_dto = plugin.KEKMetaDTO(mock.MagicMock())
        kek_meta_dto.plugin_meta = None
        return self.plugin.bind_kek_metadata(kek_meta_dto)

    def test_encrypt_unicode_raises_value_error(self):
        unencrypted = u'unicode_beer\U0001F37A'
        encrypt_dto = plugin.EncryptDTO(unencrypted)
        secret = mock.MagicMock()
        secret.mime_type = 'text/plain'
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        self.assertRaises(
            ValueError,
            self.plugin.encrypt,
            encrypt_dto,
            kek_meta_dto,
            mock.MagicMock(),
        )

    def test_encrypt_with_unicode_kek_must_pass(self):
        """Test plan:

                Generate a kek
                Encrypt with master kek
                Convert to unicode
                call plugin.encrypt on unencrypted
                decrypt response cypher_text
                Compare with unencrypted
        """
        project_kek = fernet.Fernet.generate_key()
        encryptor = fernet.Fernet(self.plugin.master_kek)
        ENC_project_kek = encryptor.encrypt(project_kek)
        UENC_project_kek = six.u(ENC_project_kek)
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        kek_meta_dto.plugin_meta = UENC_project_kek

        unencrypted = b'PlainTextSecret'
        encrypt_dto = plugin.EncryptDTO(unencrypted)
        response_dto = self.plugin.encrypt(encrypt_dto,
                                           kek_meta_dto,
                                           mock.MagicMock())

        project_encryptor = fernet.Fernet(project_kek)
        decrypted = project_encryptor.decrypt(response_dto.cypher_text)
        self.assertEqual(unencrypted, decrypted)

    def test_decrypt_kek_not_created(self):
        kek_meta_dto = mock.MagicMock()
        kek_meta_dto.plugin_meta = None
        self.assertRaises(
            ValueError,
            self.plugin.decrypt,
            mock.MagicMock(),
            kek_meta_dto,
            mock.MagicMock(),
            mock.MagicMock(),
        )

    def test_byte_string_encryption(self):
        unencrypted = b'some_secret'
        encrypt_dto = plugin.EncryptDTO(unencrypted)
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        response_dto = self.plugin.encrypt(encrypt_dto,
                                           kek_meta_dto,
                                           mock.MagicMock())
        decrypt_dto = plugin.DecryptDTO(response_dto.cypher_text)
        decrypted = self.plugin.decrypt(decrypt_dto, kek_meta_dto,
                                        response_dto.kek_meta_extended,
                                        mock.MagicMock())
        self.assertEqual(unencrypted, decrypted)

    def test_random_bytes_encryption(self):
        unencrypted = os.urandom(10)
        encrypt_dto = plugin.EncryptDTO(unencrypted)
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        response_dto = self.plugin.encrypt(encrypt_dto,
                                           kek_meta_dto,
                                           mock.MagicMock())
        decrypt_dto = plugin.DecryptDTO(response_dto.cypher_text)
        decrypted = self.plugin.decrypt(decrypt_dto, kek_meta_dto,
                                        response_dto.kek_meta_extended,
                                        mock.MagicMock())
        self.assertEqual(unencrypted, decrypted)

    def test_generate_256_bit_key(self):
        secret = models.Secret()
        secret.bit_length = 256
        secret.algorithm = "AES"
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        generate_dto = plugin.GenerateDTO(
            secret.algorithm,
            secret.bit_length,
            secret.mode, None)
        response_dto = self.plugin.generate_symmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )
        decrypt_dto = plugin.DecryptDTO(response_dto.cypher_text)
        key = self.plugin.decrypt(decrypt_dto, kek_meta_dto,
                                  response_dto.kek_meta_extended,
                                  mock.MagicMock())
        self.assertEqual(32, len(key))

    def test_generate_192_bit_key(self):
        secret = models.Secret()
        secret.bit_length = 192
        secret.algorithm = "AES"
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        generate_dto = plugin.GenerateDTO(
            secret.algorithm,
            secret.bit_length,
            None, None)
        response_dto = self.plugin.generate_symmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )
        decrypt_dto = plugin.DecryptDTO(response_dto.cypher_text)
        key = self.plugin.decrypt(decrypt_dto, kek_meta_dto,
                                  response_dto.kek_meta_extended,
                                  mock.MagicMock())
        self.assertEqual(24, len(key))

    def test_generate_128_bit_key(self):
        secret = models.Secret()
        secret.bit_length = 128
        secret.algorithm = "AES"
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        generate_dto = plugin.GenerateDTO(
            secret.algorithm,
            secret.bit_length,
            None, None)
        response_dto = self.plugin.generate_symmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )
        decrypt_dto = plugin.DecryptDTO(response_dto.cypher_text)
        key = self.plugin.decrypt(decrypt_dto, kek_meta_dto,
                                  response_dto.kek_meta_extended,
                                  mock.MagicMock())
        self.assertEqual(16, len(key))

    def test_supports_encrypt_decrypt(self):
        self.assertTrue(
            self.plugin.supports(plugin.PluginSupportTypes.ENCRYPT_DECRYPT)
        )

    def test_supports_symmetric_key_generation(self):
        self.assertTrue(
            self.plugin.supports(
                plugin.PluginSupportTypes.SYMMETRIC_KEY_GENERATION, 'AES', 64)
        )
        self.assertFalse(
            self.plugin.supports(
                plugin.PluginSupportTypes.SYMMETRIC_KEY_GENERATION, 'AES')
        )
        self.assertTrue(
            self.plugin.supports(
                plugin.PluginSupportTypes.SYMMETRIC_KEY_GENERATION,
                'hmacsha512', 128)
        )
        self.assertFalse(
            self.plugin.supports(
                plugin.PluginSupportTypes.SYMMETRIC_KEY_GENERATION,
                'hmacsha512', 12)
        )
        self.assertFalse(
            self.plugin.supports(
                plugin.PluginSupportTypes.SYMMETRIC_KEY_GENERATION,
                'Camillia', 128)
        )

    def test_does_not_support_unknown_type(self):
        self.assertFalse(
            self.plugin.supports("SOMETHING_RANDOM")
        )

    def test_bind_kek_metadata(self):
        kek_metadata_dto = mock.MagicMock()
        kek_metadata_dto = self.plugin.bind_kek_metadata(kek_metadata_dto)

        self.assertEqual('aes', kek_metadata_dto.algorithm)
        self.assertEqual(128, kek_metadata_dto.bit_length)
        self.assertEqual('cbc', kek_metadata_dto.mode)

    def test_supports_asymmetric_key_generation(self):
        self.assertTrue(
            self.plugin.supports(
                plugin.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION,
                'DSA', 1024)
        )
        self.assertTrue(
            self.plugin.supports(
                plugin.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION,
                "RSA", 1024)
        )
        self.assertFalse(
            self.plugin.supports(
                plugin.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION,
                "DSA", 512)
        )
        self.assertFalse(
            self.plugin.supports(
                plugin.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION,
                "RSA", 64)
        )

    def test_generate_asymmetric_1024_bit_key(self):
        generate_dto = plugin.GenerateDTO('rsa', 1024, None, None)
        kek_meta_dto = self._get_mocked_kek_meta_dto()

        private_dto, public_dto, passwd_dto = self.plugin.generate_asymmetric(
            generate_dto, kek_meta_dto, mock.MagicMock())

        decrypt_dto = plugin.DecryptDTO(private_dto.cypher_text)
        private_dto = self.plugin.decrypt(decrypt_dto,
                                          kek_meta_dto,
                                          private_dto.kek_meta_extended,
                                          mock.MagicMock())

        decrypt_dto = plugin.DecryptDTO(public_dto.cypher_text)
        public_dto = self.plugin.decrypt(decrypt_dto,
                                         kek_meta_dto,
                                         public_dto.kek_meta_extended,
                                         mock.MagicMock())

        # check we can reload the private and public keys
        private_key = serialization.load_pem_private_key(
            data=private_dto,
            password=None,
            backend=default_backend()
        )

        public_key = serialization.load_pem_public_key(
            data=public_dto,
            backend=default_backend()
        )

        self.assertEqual(1024, private_key.key_size)
        self.assertEqual(1024, public_key.key_size)

        public_key = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )

        # get the public key from the private key we recovered to compare
        recovered_key = private_key.public_key().public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )

        self.assertTrue(public_key == recovered_key)

    def test_generate_1024_bit_RSA_key_with_passphrase(self):
        generate_dto = plugin.GenerateDTO('rsa', 1024, None, 'changeme')
        kek_meta_dto = self._get_mocked_kek_meta_dto()

        private_dto, public_dto, passwd_dto = self.plugin.generate_asymmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )

        decrypt_dto = plugin.DecryptDTO(private_dto.cypher_text)
        private_dto = self.plugin.decrypt(decrypt_dto,
                                          kek_meta_dto,
                                          private_dto.kek_meta_extended,
                                          mock.MagicMock())

        decrypt_dto = plugin.DecryptDTO(public_dto.cypher_text)
        public_dto = self.plugin.decrypt(decrypt_dto,
                                         kek_meta_dto,
                                         public_dto.kek_meta_extended,
                                         mock.MagicMock())

        # check we can reload the private and public keys
        private_key = serialization.load_pem_private_key(
            data=private_dto,
            password='changeme'.encode(),
            backend=default_backend()
        )

        public_key = serialization.load_pem_public_key(
            data=public_dto,
            backend=default_backend()
        )

        self.assertEqual(1024, private_key.key_size)
        self.assertEqual(1024, public_key.key_size)

        public_key = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )

        # get the public key from the private key we recovered to compare
        recovered_key = private_key.public_key().public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )

        self.assertTrue(public_key == recovered_key)

    def test_generate_1024_bit_DSA_key_with_passphrase(self):
        generate_dto = plugin.GenerateDTO('dsa', 1024, None, 'changeme')
        kek_meta_dto = self._get_mocked_kek_meta_dto()

        private_dto, public_dto, passwd_dto = self.plugin.generate_asymmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )

        decrypt_dto = plugin.DecryptDTO(private_dto.cypher_text)
        private_dto = self.plugin.decrypt(decrypt_dto,
                                          kek_meta_dto,
                                          private_dto.kek_meta_extended,
                                          mock.MagicMock())

        decrypt_dto = plugin.DecryptDTO(public_dto.cypher_text)
        public_dto = self.plugin.decrypt(decrypt_dto,
                                         kek_meta_dto,
                                         public_dto.kek_meta_extended,
                                         mock.MagicMock())

        # check we can reload the private and public keys
        private_key = serialization.load_der_private_key(
            data=private_dto,
            password='changeme'.encode(),
            backend=default_backend()
        )

        public_key = serialization.load_der_public_key(
            data=public_dto,
            backend=default_backend()
        )

        self.assertEqual(1024, private_key.key_size)
        self.assertEqual(1024, public_key.key_size)

        public_key = public_key.public_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        )

        # get the public key from the private key we recovered to compare
        recovered_key = private_key.public_key().public_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        )

        self.assertTrue(public_key == recovered_key)

    def test_generate_1024_DSA_key_in_pem_and_reconstruct_key_der(self):
        generate_dto = plugin.GenerateDTO('dsa', 1024, None, None)
        kek_meta_dto = self._get_mocked_kek_meta_dto()

        private_dto, public_dto, passwd_dto = self.plugin.generate_asymmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )

        decrypt_dto = plugin.DecryptDTO(private_dto.cypher_text)
        private_dto = self.plugin.decrypt(decrypt_dto,
                                          kek_meta_dto,
                                          private_dto.kek_meta_extended,
                                          mock.MagicMock())

        private_key = serialization.load_der_private_key(
            data=private_dto,
            password=None,
            backend=default_backend()
        )

        self.assertEqual(1024, private_key.key_size)

    def test_generate_128_bit_hmac_key(self):
        secret = models.Secret()
        secret.bit_length = 128
        secret.algorithm = "hmacsha256"
        kek_meta_dto = self._get_mocked_kek_meta_dto()
        generate_dto = plugin.GenerateDTO(
            secret.algorithm,
            secret.bit_length,
            None, None)
        response_dto = self.plugin.generate_symmetric(
            generate_dto,
            kek_meta_dto,
            mock.MagicMock()
        )
        decrypt_dto = plugin.DecryptDTO(response_dto.cypher_text)
        key = self.plugin.decrypt(decrypt_dto, kek_meta_dto,
                                  response_dto.kek_meta_extended,
                                  mock.MagicMock())
        self.assertEqual(16, len(key))

    def test_get_plugin_name(self):
        self.assertIsNotNone(self.plugin.get_plugin_name())



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest import mock

from oslo_config import cfg
import six

from barbican.common import config
from barbican.common import utils
from barbican.tests import utils as test_utils


class WhenTestingHostnameForRefsGetter(test_utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingHostnameForRefsGetter, self).setUp()

        self.host = 'host'
        self.version = 'version'
        self.external_project_id = 'external_project_id'
        self.resource = 'resource'

        self._old_host = utils.CONF.host_href
        self._old_version = utils.API_VERSION
        utils.CONF.set_override('host_href', self.host)
        utils.API_VERSION = self.version

    def tearDown(self):
        super(WhenTestingHostnameForRefsGetter, self).tearDown()
        utils.CONF.clear_override('host_href')
        utils.API_VERSION = self._old_version

    def test_hostname_for_refs(self):
        uri = utils.hostname_for_refs(resource=self.resource)
        self.assertEqual("{0}/{1}/{2}".format(self.host, self.version,
                                              self.resource), uri)

    def test_hostname_for_refs_no_resource(self):
        uri = utils.hostname_for_refs()
        self.assertEqual("{0}/{1}".format(self.host, self.version), uri)


class WhenTestingHostByWsgiRequestForRefsGetter(test_utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingHostByWsgiRequestForRefsGetter, self).setUp()

        self._old_version = utils.API_VERSION
        self.host = 'http://my_host:9311'
        self.version = 'version1'
        self.external_project_id = 'external_project_id'
        self.resource = 'my_resource'

        test_utils.mock_pecan_request(self, host=self.host)

        utils.CONF.set_override('host_href', None)
        utils.API_VERSION = self.version

    def tearDown(self):
        super(WhenTestingHostByWsgiRequestForRefsGetter, self).tearDown()
        utils.CONF.clear_override('host_href')
        utils.API_VERSION = self._old_version

    def test_hostname_for_refs(self):
        uri = utils.hostname_for_refs(resource=self.resource)
        self.assertEqual("{0}/{1}/{2}".format(self.host, self.version,
                                              self.resource), uri)

    def test_blank_conf_hosthref_for_refs(self):
        utils.CONF.set_override('host_href', '')
        uri = utils.hostname_for_refs(resource=self.resource)
        self.assertEqual("{0}/{1}/{2}".format(self.host, self.version,
                                              self.resource), uri)

    def test_hostname_for_refs_no_resource(self):
        uri = utils.hostname_for_refs()
        self.assertEqual("{0}/{1}".format(self.host, self.version), uri)


class WhenTestingAcceptEncodingGetter(test_utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingAcceptEncodingGetter, self).setUp()

        self.req = mock.Mock()

    def test_parses_accept_encoding_header(self):
        self.req.get_header.return_value = '*'
        ae = utils.get_accepted_encodings(self.req)
        self.req.get_header.assert_called_once_with('Accept-Encoding')
        self.assertEqual(['*'], ae)

    def test_returns_none_for_empty_encoding(self):
        self.req.get_header.return_value = None
        ae = utils.get_accepted_encodings(self.req)
        self.assertIsNone(ae)

    def test_parses_single_accept_with_quality_value(self):
        self.req.get_header.return_value = 'base64;q=0.7'
        ae = utils.get_accepted_encodings(self.req)
        self.assertEqual(['base64'], ae)

    def test_parses_more_than_one_encoding(self):
        self.req.get_header.return_value = 'base64, gzip'
        ae = utils.get_accepted_encodings(self.req)
        self.assertEqual(['base64', 'gzip'], ae)

    def test_can_sort_by_quality_value(self):
        self.req.get_header.return_value = 'base64;q=0.5, gzip;q=0.6, compress'
        ae = utils.get_accepted_encodings(self.req)
        self.assertEqual(['compress', 'gzip', 'base64'], ae)

    def test_returns_none_on_invalid_quality_type(self):
        self.req.get_header.return_value = 'base64;q=three'
        ae = utils.get_accepted_encodings(self.req)
        self.assertIsNone(ae)

    def test_returns_none_on_quality_too_large(self):
        self.req.get_header.return_value = 'base64;q=1.1'
        ae = utils.get_accepted_encodings(self.req)
        self.assertIsNone(ae)

    def test_returns_none_on_quality_too_small(self):
        self.req.get_header.return_value = 'base64;q=-0.1'
        ae = utils.get_accepted_encodings(self.req)
        self.assertIsNone(ae)

    def test_ignores_encoding_with_zero_quality_value(self):
        self.req.get_header.return_value = 'base64;q=0.5, gzip;q=0.0, compress'
        ae = utils.get_accepted_encodings(self.req)
        self.assertEqual(['compress', 'base64'], ae)


class WhenTestingGenerateFullClassnameForInstance(test_utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingGenerateFullClassnameForInstance, self).setUp()

        self.instance = test_utils.DummyClassForTesting()

    def test_get_fullname_for_null_instance_raises_exception(self):
        self.assertRaises(ValueError, utils.generate_fullname_for, None)

    def test_get_fullname_for_string_doesnt_include_module(self):
        test_string = "foo"
        fullname = utils.generate_fullname_for(test_string)
        self.assertEqual(0, fullname.count("."))
        self.assertNotIn(six.moves.builtins.__name__, fullname)

    def test_returns_class_name_on_null_module(self):
        self.instance.__class__.__module__ = None
        name = utils.generate_fullname_for(self.instance)
        self.assertEqual('DummyClassForTesting', name)

    def test_returns_qualified_name(self):
        self.instance.__class__.__module__ = 'dummy'
        name = utils.generate_fullname_for(self.instance)
        self.assertEqual('dummy.DummyClassForTesting', name)


class TestConfigValues(test_utils.BaseTestCase):

    def setUp(self):
        super(TestConfigValues, self).setUp()
        self.barbican_config = config.CONF
        self.oslo_config = cfg.CONF

    def test_barbican_conf_values_made_visible_to_oslo_conf(self):
        """In this, checking oslo CONF values are same as barbican config

        This tests shows that after the change values referenced via
        oslo_config.cfg.CONF value are same as
        barbican.common.config.CONF.
        """

        # Checking that 'admin_role' value referred via
        # barbican.common.config.CONF is same as oslo_config.cfg.CONF
        self.assertEqual('admin', self.barbican_config._get('admin_role'))
        self.assertEqual('admin', self.barbican_config.admin_role)

        self.assertEqual('admin', self.oslo_config._get('admin_role'))
        self.assertEqual('admin', self.oslo_config.admin_role)

        # No error in getting 'project' value from both config reading
        # mechanism
        self.assertEqual('barbican', self.barbican_config.project)
        self.assertEqual('barbican', self.oslo_config.project)



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.common import exception
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repos
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class ProjectSecretStore(base.BarbicanObject, base.BarbicanPersistentObject,
                         object_base.VersionedObjectDictCompat):
    fields = {
        'secret_store_id': fields.StringField(nullable=True, default=None),
        'project_id': fields.StringField(nullable=True, default=None),
        'secret_store': fields.ObjectField('SecretStores',
                                           nullable=True, default=None),
        'project': fields.ObjectField('Project', nullable=True, default=None),
        'status': fields.StringField(nullable=True, default=base.States.ACTIVE)
    }

    db_model = models.ProjectSecretStore
    db_repo = repos.get_project_secret_store_repository()
    synthetic_fields = ['secret_store', 'project']

    def _validate_fields(self, change_fields):

        msg = u._("Must supply non-None {0} argument for ProjectSecretStore "
                  " entry.")

        if not change_fields.get('project_id'):
            raise exception.MissingArgumentError(msg.format("project_id"))
        if not change_fields.get('secret_store_id'):
            raise exception.MissingArgumentError(msg.format("secret_store_id"))

    @classmethod
    def get_secret_store_for_project(cls, project_id, external_project_id,
                                     suppress_exception=False, session=None):
        pss_db = cls.db_repo.get_secret_store_for_project(
            project_id,
            external_project_id,
            suppress_exception,
            session)
        return cls()._from_db_object(pss_db)

    @classmethod
    def create_or_update_for_project(cls, project_id, secret_store_id,
                                     session=None):
        pss_db = cls.db_repo.create_or_update_for_project(project_id,
                                                          secret_store_id,
                                                          session)

        return cls()._from_db_object(pss_db)

    @classmethod
    def get_count_by_secret_store(cls, secret_store_id, session=None):
        number_pss = cls.db_repo.get_count_by_secret_store(secret_store_id,
                                                           session)
        return number_pss



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import six

from barbican.common import exception
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils as utils


class WhenTestingContainerConsumerRepository(utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingContainerConsumerRepository, self).setUp()
        self.repo = repositories.ContainerConsumerRepo()
        self.repo_container = repositories.ContainerRepo()

    def test_should_update_with_duplicate_consumer(self):
        session = self.repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        container = models.Container()
        container.project_id = project.id
        container.save(session=session)

        # Create a consumer.
        consumer = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name', 'URL': 'www.foo.com'})
        consumer.save(session=session)

        # Commit things so far, because the 'create_or_update_from' call below
        # will handle consumer metadata with same composite key items already
        # existing, and then rollback this session's transaction, which would
        # remove the items added above and result in a not-found error below.
        session.commit()

        # Try to create a consumer on the container...should re-use the
        # one added above.
        consumer2 = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name', 'URL': 'www.foo.com'})
        self.repo.create_or_update_from(consumer2, container, session=session)

        container2 = self.repo_container.get(
            container.id, project.external_id, session=session)
        self.assertEqual(1, len(container2.consumers))

    def test_should_raise_constraint_create_same_composite_key_no_id(self):
        session = self.repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        container = models.Container()
        container.project_id = project.id
        container.save(session=session)

        # Create a consumer.
        consumer = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name', 'URL': 'www.foo.com'})
        consumer.save(session=session)

        # Commit things so far, because the 'create_from' call below will
        # handle consumer metadata with same composite key items already
        # existing, and then rollback this session's transaction, which would
        # remove the items added above and result in a not-found error below.
        session.commit()

        # Create a new entity with the same composite key as the first one.
        consumer2 = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name', 'URL': 'www.foo.com'})

        exception_result = self.assertRaises(
            exception.ConstraintCheck,
            self.repo.create_from,
            consumer2,
            session=session)
        self.assertIn(
            "SQL constraint check failed",
            six.text_type(exception_result))

    def test_should_raise_no_result_found_get_container_id(self):
        session = self.repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.repo.get_by_container_id,
            "my container id",
            session=session,
            suppress_exception=False)

    def test_should_raise_no_result_found_get_by_values_no_deleted(self):
        session = self.repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.repo.get_by_values,
            "my container id",
            "name",
            "url",
            session=session,
            suppress_exception=False,
            show_deleted=False)

    def test_should_raise_no_result_found_get_by_values_show_deleted(self):
        session = self.repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.repo.get_by_values,
            "my container id",
            "name",
            "url",
            session=session,
            suppress_exception=False,
            show_deleted=True)

    def test_should_get_count_zero(self):
        session = self.repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        session.commit()
        count = self.repo.get_count(project.id, session=session)

        self.assertEqual(0, count)

    def test_should_get_count_one(self):
        session = self.repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        container = models.Container()
        container.project_id = project.id
        container.save(session=session)

        consumer = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name', 'URL': 'www.foo.com'})
        consumer.save(session=session)
        session.commit()

        count = self.repo.get_count(project.id, session=session)
        self.assertEqual(1, count)

    def test_should_get_count_one_after_delete(self):
        session = self.repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        container = models.Container()
        container.project_id = project.id
        container.save(session=session)

        consumer = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name1', 'URL': 'www.foo.com'})
        consumer.save(session=session)

        consumer = models.ContainerConsumerMetadatum(
            container.id, project.id, {'name': 'name2', 'URL': 'www.foo.com'})
        consumer.save(session=session)
        session.commit()

        count = self.repo.get_count(project.id, session=session)
        self.assertEqual(2, count)

        self.repo.delete_entity_by_id(consumer.id, "my keystone id",
                                      session=session)
        session.commit()

        count = self.repo.get_count(project.id, session=session)
        self.assertEqual(1, count)



# Copyright (c) 2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from testtools import testcase

from barbican.tests import utils
from functionaltests.api import base
from functionaltests.api.v1.behaviors import container_behaviors
from functionaltests.api.v1.behaviors import secret_behaviors
from functionaltests.api.v1.models import container_models
from functionaltests.api.v1.models import secret_models

create_secret_defaults_data = {
    "name": "AES key",
    "expiration": "2030-02-28T19:14:44.180394",
    "algorithm": "aes",
    "bit_length": 256,
    "mode": "cbc",
    "payload": b"gF6+lLoF3ohA9aPRpt+6bQ==",
    "payload_content_type": "application/octet-stream",
    "payload_content_encoding": "base64",
}

create_container_defaults_data = {
    "name": "containername",
    "type": "generic",
    "secret_refs": [
        {
            "name": "secret1",
        },
        {
            "name": "secret2",
        },
        {
            "name": "secret3"
        }
    ]
}

create_container_rsa_data = {
    "name": "rsacontainer",
    "type": "rsa",
    "secret_refs": [
        {
            "name": "public_key",
        },
        {
            "name": "private_key",
        },
        {
            "name": "private_key_passphrase"
        }
    ]
}

create_container_empty_data = {
    "name": None,
    "type": "generic",
    "secret_refs": []
}


@utils.parameterized_test_case
class ContainersTestCase(base.TestCase):

    def setUp(self):
        super(ContainersTestCase, self).setUp()
        self.secret_behaviors = secret_behaviors.SecretBehaviors(self.client)
        self.behaviors = container_behaviors.ContainerBehaviors(
            self.client)

        # Set up three secrets
        secret_ref_1 = self._create_a_secret()
        secret_ref_2 = self._create_a_secret()
        secret_ref_3 = self._create_a_secret()

        create_container_defaults_data[
            'secret_refs'][0]['secret_ref'] = secret_ref_1
        create_container_defaults_data[
            'secret_refs'][1]['secret_ref'] = secret_ref_2
        create_container_defaults_data[
            'secret_refs'][2]['secret_ref'] = secret_ref_3

        create_container_rsa_data[
            'secret_refs'][0]['secret_ref'] = secret_ref_1
        create_container_rsa_data[
            'secret_refs'][1]['secret_ref'] = secret_ref_2
        create_container_rsa_data[
            'secret_refs'][2]['secret_ref'] = secret_ref_3

        self.secret_id_1 = secret_ref_1.split('/')[-1]
        self.secret_id_2 = secret_ref_2.split('/')[-1]
        self.secret_id_3 = secret_ref_3.split('/')[-1]

    def tearDown(self):
        self.secret_behaviors.delete_all_created_secrets()
        self.behaviors.delete_all_created_containers()
        super(ContainersTestCase, self).tearDown()

    def _create_a_secret(self):
        secret_model = secret_models.SecretModel(**create_secret_defaults_data)
        resp, secret_ref = self.secret_behaviors.create_secret(secret_model)
        self.assertEqual(201, resp.status_code)
        self.assertIsNotNone(secret_ref)

        return secret_ref

    def _get_a_secret(self, secret_id):
        resp = self.client.get('secrets/{0}'.format(secret_id))
        self.assertEqual(200, resp.status_code)
        return resp.json()

    @testcase.attr('positive')
    def test_container_create_empty(self):
        """Covers creating an empty generic container."""
        test_model = container_models.ContainerModel(
            **create_container_empty_data)

        resp, container_ref = self.behaviors.create_container(
            test_model)
        self.assertEqual(201, resp.status_code)
        self.assertGreater(len(container_ref), 0)

    @testcase.attr('positive')
    def test_container_create_defaults(self):
        """Covers creating a container with three secret refs."""
        test_model = container_models.ContainerModel(
            **create_container_defaults_data)

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)
        self.assertGreater(len(container_ref), 0)

    @testcase.attr('positive')
    def test_container_create_rsa(self):
        """Create an RSA container with expected secret refs."""
        test_model = container_models.ContainerModel(
            **create_container_rsa_data)

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)
        self.assertGreater(len(container_ref), 0)

    @utils.parameterized_dataset({
        'alphanumeric': ['a2j3j6ll9'],
        'punctuation': ['~!@#$%^&*()_+`-={}[]|:;<>,.?'],
        'len_255': [base.TestCase.max_sized_field],
        'uuid': ['54262d9d-4bc7-4821-8df0-dc2ca8e112bb'],
        'empty': ['']
    })
    @testcase.attr('positive')
    def test_container_get_defaults_w_valid_name(self, name):
        """Covers getting a generic container with a three secrets."""
        test_model = container_models.ContainerModel(
            **create_container_defaults_data)
        overrides = {'name': name}
        test_model.override_values(**overrides)

        secret_refs = []
        for secret_ref in test_model.secret_refs:
            secret_refs.append(secret_ref['secret_ref'])

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)
        self.assertGreater(len(container_ref), 0)

        get_resp = self.behaviors.get_container(container_ref)

        # Verify the response data
        self.assertEqual(200, get_resp.status_code)
        self.assertEqual(test_model.name, get_resp.model.name)
        self.assertEqual(container_ref, get_resp.model.container_ref)
        self.assertEqual(test_model.type, get_resp.model.type)

        # Verify the secret refs in the response
        self.assertEqual(3, len(get_resp.model.secret_refs))
        self.assertIn(get_resp.model.secret_refs[0].secret_ref, secret_refs)
        self.assertIn(get_resp.model.secret_refs[1].secret_ref, secret_refs)
        self.assertIn(get_resp.model.secret_refs[2].secret_ref, secret_refs)

    @testcase.attr('positive')
    def test_container_get_rsa(self):
        """Covers getting an rsa container."""
        test_model = container_models.ContainerModel(
            **create_container_rsa_data)
        secret_refs = []
        for secret_ref in test_model.secret_refs:
            secret_refs.append(secret_ref['secret_ref'])

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)
        self.assertGreater(len(container_ref), 0)

        get_resp = self.behaviors.get_container(
            container_ref)

        # Verify the response data
        self.assertEqual(200, get_resp.status_code)
        self.assertEqual("rsacontainer", get_resp.model.name)
        self.assertEqual(container_ref, get_resp.model.container_ref)
        self.assertEqual("rsa", get_resp.model.type)

        # Verify the secret refs in the response
        self.assertEqual(3, len(get_resp.model.secret_refs))
        self.assertIn(get_resp.model.secret_refs[0].secret_ref, secret_refs)
        self.assertIn(get_resp.model.secret_refs[1].secret_ref, secret_refs)
        self.assertIn(get_resp.model.secret_refs[2].secret_ref, secret_refs)

    @testcase.attr('positive')
    def test_containers_get_defaults(self):
        """Covers getting a list of containers."""
        limit = 10
        offset = 0
        test_model = container_models.ContainerModel(
            **create_container_defaults_data)
        for i in range(11):
            resp, container_ref = self.behaviors.create_container(test_model)
            self.assertEqual(201, resp.status_code)
            self.assertGreater(len(container_ref), 0)

        resp, containers, next_ref, prev_ref = self.behaviors.get_containers(
            limit=limit,
            offset=offset
        )

        self.assertEqual(200, resp.status_code)
        self.assertEqual(limit, len(containers))
        self.assertIsNone(prev_ref)
        self.assertIsNotNone(next_ref)

    def test_container_delete_defaults(self):
        """Covers deleting a container."""
        test_model = container_models.ContainerModel(
            **create_container_defaults_data)

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)
        self.assertGreater(len(container_ref), 0)

        del_resp = self.behaviors.delete_container(container_ref)
        self.assertEqual(204, del_resp.status_code)
        self.assertEqual(0, len(del_resp.content))



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import six

from oslo_utils import uuidutils

from barbican.common import exception
from barbican import objects
from barbican.tests.objects import test_ovo_base


class TestProjectSecretStore(test_ovo_base.OVOTestCase):
    def setUp(self):
        super(TestProjectSecretStore, self).setUp()
        self.init()

    def init(self):
        self.def_name = "PKCS11 HSM"
        self.def_store_plugin = "store_crypto"
        self.def_crypto_plugin = "p11_crypto"
        self.default_secret_stores = self._create_secret_store_obj(
            self.def_name, self.def_store_plugin, self.def_crypto_plugin, True)

    def _create_secret_store_obj(self, name, store_plugin, crypto_plugin=None,
                                 global_default=None):
        secret_stores_obj = objects.SecretStores(name=name,
                                                 store_plugin=store_plugin,
                                                 crypto_plugin=crypto_plugin,
                                                 global_default=global_default)
        secret_stores_obj.create(session=self.session)
        return secret_stores_obj

    def _create_project(self):
        external_id = 'keystone_project_id' + uuidutils.generate_uuid(
            dashed=True)
        project = objects.Project(external_id=external_id)
        project.create(session=self.session)
        return project

    def _create_project_secret_store(self, project_id, secret_store_id):
        project_secret_store = objects.ProjectSecretStore(
            project_id=project_id,
            secret_store_id=secret_store_id)
        project_secret_store.create(session=self.session)
        return project_secret_store

    def test_ovo_create_by_entity_id(self):
        """Tests for 'create' call by project secret store id"""

        project = self._create_project()

        project_secret_store = self._create_project_secret_store(
            project.id, self.default_secret_stores.id)

        self.assertIsNotNone(project_secret_store)
        self.assertEqual(project.id, project_secret_store.project_id)
        self.assertEqual(self.default_secret_stores.id,
                         project_secret_store.secret_store_id)
        self.assertEqual(objects.States.ACTIVE, project_secret_store.status)
        # assert values via relationship
        self.assertEqual(self.default_secret_stores.store_plugin,
                         project_secret_store.secret_store.store_plugin)
        self.assertEqual(project.external_id,
                         project_secret_store.project.external_id)

    def test_ovo_should_raise_notfound_exception_get_by_entity_id(self):
        self.assertRaises(exception.NotFound, objects.ProjectSecretStore.get,
                          "invalid_id", suppress_exception=False)

    def test_ovo_delete_entity_by_id(self):

        project = self._create_project()

        project_secret_store = self._create_project_secret_store(
            project.id, self.default_secret_stores.id)

        project_secret_store = objects.ProjectSecretStore.get(
            project_secret_store.id, session=self.session)

        self.assertIsNotNone(project_secret_store)

        objects.ProjectSecretStore.delete_entity_by_id(
            project_secret_store.id, None, session=self.session)
        project_secret_store = objects.ProjectSecretStore.get(
            project_secret_store.id,
            suppress_exception=True,
            session=self.session)

        self.assertIsNone(project_secret_store)

    def test_ovo_should_raise_constraint_for_same_project_id(self):
        """Check preferred secret store is set only once for project"""

        project1 = self._create_project()
        name = "first_name"
        store_plugin = 'first_store'
        crypto_plugin = 'first_crypto'
        secret_stores1 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        # set preferred secret store for project1
        self._create_project_secret_store(project1.id,
                                          secret_stores1.id)

        name = "second_name"
        store_plugin = 'second_store'
        crypto_plugin = 'second_crypto'
        secret_stores2 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        self.assertRaises(exception.ConstraintCheck,
                          self._create_project_secret_store,
                          project1.id, secret_stores2.id)

    def test_ovo_do_entity_name(self):
        """Code coverage for entity_name which is used in case of exception.

        Raising duplicate error when try to set another entry for existing
        project
        """
        project1 = self._create_project()
        name = "first name"
        store_plugin = 'first_store'
        crypto_plugin = 'first_crypto'
        secret_stores1 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        # set preferred secret store for project1
        self._create_project_secret_store(project1.id,
                                          secret_stores1.id)
        try:
            name = "second_name"
            store_plugin = 'second_store'
            crypto_plugin = 'second_crypto'
            secret_stores2 = self._create_secret_store_obj(name,
                                                           store_plugin,
                                                           crypto_plugin,
                                                           False)
            self._create_project_secret_store(project1.id, secret_stores2.id)
            self.assertFail()
        except exception.ConstraintCheck as ex:
            self.assertIn("SQL constraint check failed", six.text_type(ex))

    def test_ovo_get_secret_store_for_project(self):
        project1 = self._create_project()
        name = "first_name"
        store_plugin = 'first_store'
        crypto_plugin = 'first_crypto'
        secret_stores1 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        # set preferred secret store for project1
        project_secret_store = self._create_project_secret_store(
            project1.id, secret_stores1.id)

        # get preferred secret store by barbican project id
        read_project_secret_stores = objects.ProjectSecretStore.\
            get_secret_store_for_project(project1.id, None,
                                         session=self.session)

        self.assertEqual(project_secret_store.project_id,
                         read_project_secret_stores.project_id)
        self.assertEqual(project_secret_store.secret_store_id,
                         read_project_secret_stores.secret_store_id)

        # get preferred secret store by keystone project id
        read_project_secret_stores = objects.ProjectSecretStore. \
            get_secret_store_for_project(None, project1.external_id,
                                         session=self.session)

        self.assertEqual(project_secret_store.project_id,
                         read_project_secret_stores.project_id)
        self.assertEqual(project1.external_id,
                         read_project_secret_stores.project.external_id)

        self.assertEqual(project_secret_store.secret_store_id,
                         read_project_secret_stores.secret_store_id)

    def test_ovo_raise_notfound_exception_get_secret_store_for_project(self):
        self.assertRaises(
            exception.NotFound,
            objects.ProjectSecretStore.get_secret_store_for_project,
            "invalid_id", None, suppress_exception=False)

    def test_ovo_with_exception_suppressed_get_secret_store_for_project(self):
        returned_value = objects.ProjectSecretStore. \
            get_secret_store_for_project("invalid_id", None,
                                         suppress_exception=True,
                                         session=self.session)
        self.assertIsNone(returned_value)

    def test_ovo_get_project_entities(self):
        entities = objects.ProjectSecretStore.get_project_entities(
            uuidutils.generate_uuid(dashed=False), session=self.session)
        self.assertEqual([], entities)

    def test_ovo_create_or_update_for_project(self):
        project1 = self._create_project()
        name = "first_name"
        store_plugin = 'first_store'
        crypto_plugin = 'first_crypto'
        secret_stores1 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        # assert that no preferred secret store is set project.
        entity = objects.ProjectSecretStore.get_secret_store_for_project(
            project1.id, None, suppress_exception=True, session=self.session)
        self.assertIsNone(entity)

        # create/set preferred secret store now
        created_entity = \
            objects.ProjectSecretStore.create_or_update_for_project(
                project1.id, secret_stores1.id, session=self.session)

        entity = objects.ProjectSecretStore.get_secret_store_for_project(
            project1.id, None, suppress_exception=False, session=self.session)
        self.assertIsNotNone(entity)  # new preferred secret store

        self.assertEqual(project1.id, entity.project_id)
        self.assertEqual(secret_stores1.id, entity.secret_store_id)
        self.assertEqual(store_plugin, entity.secret_store.store_plugin)
        self.assertEqual(crypto_plugin, entity.secret_store.crypto_plugin)
        self.assertEqual(name, entity.secret_store.name)

        name = 'second_name'
        store_plugin = 'second_store'
        crypto_plugin = 'second_crypto'
        secret_stores2 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        updated_entity = \
            objects.ProjectSecretStore.create_or_update_for_project(
                project1.id, secret_stores2.id, session=self.session)

        self.assertEqual(created_entity.id, updated_entity.id)
        self.assertEqual(secret_stores2.id, updated_entity.secret_store_id)

    def test_ovo_get_count_by_secret_store(self):
        project1 = self._create_project()
        name = "first_name"
        store_plugin = 'first_store'
        crypto_plugin = 'first_crypto'
        secret_stores1 = self._create_secret_store_obj(name, store_plugin,
                                                       crypto_plugin, False)

        count = objects.ProjectSecretStore.get_count_by_secret_store(
            secret_stores1.id, session=self.session)
        self.assertEqual(0, count)

        # create/set preferred secret store now
        objects.ProjectSecretStore.create_or_update_for_project(
            project1.id, secret_stores1.id, session=self.session)

        count = objects.ProjectSecretStore.get_count_by_secret_store(
            secret_stores1.id, session=self.session)
        self.assertEqual(1, count)

        project2 = self._create_project()
        objects.ProjectSecretStore.create_or_update_for_project(
            project2.id, secret_stores1.id, session=self.session)

        count = objects.ProjectSecretStore.get_count_by_secret_store(
            secret_stores1.id, session=self.session)
        self.assertEqual(2, count)

    def test_ovo_should_throw_exception_missing_project_id(self):
        project_secret_store_1 = objects.ProjectSecretStore(
            project_id=None, secret_store_id='ss_123456')
        project_secret_store_2 = objects.ProjectSecretStore(
            project_id='', secret_store_id='ss_123456')
        self.assertRaises(exception.MissingArgumentError,
                          project_secret_store_1.create, session=self.session)
        self.assertRaises(exception.MissingArgumentError,
                          project_secret_store_2.create, session=self.session)

    def test_ovo_should_throw_exception_missing_secret_store_id(self):
        project_secret_store_1 = objects.ProjectSecretStore(
            project_id='proj_123456', secret_store_id=None)
        project_secret_store_2 = objects.ProjectSecretStore(
            project_id='proj_123456', secret_store_id='')

        self.assertRaises(exception.MissingArgumentError,
                          project_secret_store_1.create, session=self.session)
        self.assertRaises(exception.MissingArgumentError,
                          project_secret_store_2.create, session=self.session)



"""
Copyright 2014-2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging
import os
import six


class BaseBehaviors(object):

    def __init__(self, client):
        self.LOG = logging.getLogger(type(self).__name__)
        self.client = client
        self.created_entities = []

    def get_json(self, response):
        json_data = dict()

        try:
            json_data = response.json()
        except ValueError as e:
            self.LOG.exception(e)
            self.LOG.error("Error converting response to JSON: %s",
                           six.text_type(e))
            self.LOG.error("Response Content: %s", response.content)

        return json_data

    def get_id_from_href(self, href):
        """Returns the id from reference.

        The id must be the last item in the href.

        :param href: The href containing the id.
        :returns the id portion of the href
        """

        item_id = None
        if href and len(href) > 0:
            base, item_id = os.path.split(href)
        return item_id

    def get_user_id_from_name(self, user_name):
        """From a configured user name, get the unique user id from keystone"""
        return self.client.get_user_id_from_name(user_name)

    def get_project_id_from_name(self, user_name):
        """From a configured user name, get the project id from keystone"""
        return self.client.get_project_id_from_name(user_name)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import threading
from unittest import mock

from barbican.common import utils as common_utils
from barbican.plugin.crypto import base
from barbican.plugin.crypto import manager as cm
from barbican.tests import utils


class MyThread(threading.Thread):
    def __init__(self, index, results):
        threading.Thread.__init__(self)
        self.index = index
        self.results = results

    def run(self):
        self.results[self.index] = cm.get_manager()


class WhenTestingManager(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingManager, self).setUp()

        self.plugin_returned = mock.MagicMock()
        self.plugin_type = base.PluginSupportTypes.ENCRYPT_DECRYPT
        self.plugin_returned.supports.return_value = True
        self.plugin_name = common_utils.generate_fullname_for(
            self.plugin_returned)
        self.plugin_loaded = mock.MagicMock(obj=self.plugin_returned)
        self.manager = cm.get_manager()
        self.manager.extensions = [self.plugin_loaded]

    def test_can_override_enabled_plugins(self):
        """Verify can override default configuration for plugin selection."""
        # Reset manager singleton otherwise we have test execution
        # order problems
        cm._PLUGIN_MANAGER = None

        cm.CONF.set_override(
            "enabled_crypto_plugins",
            ['foo_plugin'],
            group='crypto')

        manager_to_test = cm.get_manager()

        self.assertIsInstance(
            manager_to_test, cm._CryptoPluginManager)

        self.assertListEqual(['foo_plugin'],
                             manager_to_test._names)

    def test_get_plugin_store_generate(self):
        self.assertEqual(
            self.plugin_returned,
            self.manager.get_plugin_store_generate(self.plugin_type))

    def test_raises_error_with_wrong_plugin_type(self):
        self.plugin_returned.supports.return_value = False
        self.assertRaises(
            base.CryptoPluginUnsupportedOperation,
            self.manager.get_plugin_store_generate,
            self.plugin_type)

    def test_raises_error_with_no_active_store_generate_plugin(self):
        self.manager.extensions = []
        self.assertRaises(
            base.CryptoPluginNotFound,
            self.manager.get_plugin_store_generate,
            self.plugin_type)

    def test_get_plugin_retrieve(self):
        self.assertEqual(
            self.plugin_returned,
            self.manager.get_plugin_retrieve(self.plugin_name))

    def test_raises_error_with_wrong_plugin_name(self):
        self.assertRaises(
            base.CryptoPluginUnsupportedOperation,
            self.manager.get_plugin_retrieve,
            'other-name')

    def test_raises_error_with_no_active_plugin_name(self):
        self.manager.extensions = []
        self.assertRaises(
            base.CryptoPluginNotFound,
            self.manager.get_plugin_retrieve,
            self.plugin_name)

    def test_get_manager_with_multi_threads(self):
        self.manager.extensions = []
        self.manager = None
        results = [None] * 10
        # setup 10 threads to call get_manager() at same time
        for i in range(10):
            t = MyThread(i, results)
            t.start()
        # verify all threads return one and same plugin manager
        for i in range(10):
            self.assertIsInstance(results[i], cm._CryptoPluginManager)
            self.assertEqual(results[0], results[i])



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_policy import policy


rules = [
    policy.DocumentedRuleDefault(
        name='secret_meta:get',
        check_str='rule:all_but_audit',
        scope_types=[],
        description='metadata/: Lists a secrets user-defined metadata. || ' +
                    'metadata/{key}: Retrieves a secrets user-added metadata.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata',
                'method': 'GET'
            },
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name='secret_meta:post',
        check_str='rule:admin_or_creator',
        scope_types=[],
        description='Adds a new key/value pair to the secrets user-defined ' +
                    'metadata.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'POST'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name='secret_meta:put',
        check_str='rule:admin_or_creator',
        scope_types=[],
        description='metadata/: Sets the user-defined metadata for a secret ' +
                    '|| metadata/{key}: Updates an existing key/value pair ' +
                    'in the secrets user-defined metadata.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata',
                'method': 'PUT'
            },
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'PUT'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name='secret_meta:delete',
        check_str='rule:admin_or_creator',
        scope_types=[],
        description='Delete secret user-defined metadata by key.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'DELETE'
            }
        ]
    ),
]


def list_rules():
    return rules



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Shared business logic.
"""
from barbican.common import exception
from barbican.common import utils
from barbican.model import models
from barbican.model import repositories


LOG = utils.getLogger(__name__)

GLOBAL_PREFERRED_PROJECT_ID = "GLOBAL_PREFERRED"


def get_or_create_global_preferred_project():
    return get_or_create_project(GLOBAL_PREFERRED_PROJECT_ID)


def get_or_create_project(project_id):
    """Returns project with matching project_id.

    Creates it if it does not exist.
    :param project_id: The external-to-Barbican ID for this project.
    :param project_repo: Project repository.
    :return: Project model instance
    """
    project_repo = repositories.get_project_repository()
    project = project_repo.find_by_external_project_id(project_id,
                                                       suppress_exception=True)
    if not project:
        LOG.debug('Creating project for %s', project_id)
        project = models.Project()
        project.external_id = project_id
        project.status = models.States.ACTIVE
        try:
            project_repo.create_from(project)
        except exception.ConstraintCheck:
            # catch race condition for when another thread just created one
            project = project_repo.find_by_external_project_id(
                project_id,
                suppress_exception=False)

    return project
